% fn_clean_cpi.m
% Function to clean CPI series by removing vintages that only adjust 
% old seasonal factors and do not contain new data.
%  [cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt]=...
%   fn_clean_cpi(cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt);
%
% The function loads the raw data and deletes columns that only revise
% old seasonal factors.  The function also shortens the longer series 
% to match the length of the shortest series.
%
% Inputs:
% cpi_x - The original CPI-based data
%
% Outputs:
% cpi_x - The new cleaned data
%
function [cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt]=...
   fn_clean_cpi(cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt);
%
% remove pure seasonals; next release comes out shortly afterward
cpi_rt=sub_fn_clean1_cpi(cpi_rt);
cpi_core_rt=sub_fn_clean1_cpi(cpi_core_rt);
cpi_food_rt=sub_fn_clean1_cpi(cpi_food_rt);
cpi_gasoline_rt=sub_fn_clean1_cpi(cpi_gasoline_rt);
% remove extra observations to match the shortest available series
i1=size(cpi_rt,2);
i2=size(cpi_core_rt,2);
i3=size(cpi_food_rt,2);
i4=size(cpi_gasoline_rt,2);
[ii,pp]=min([i1,i2,i3,i4]);
if pp==1;
	[cpi_core_rt,cpi_food_rt,cpi_gasoline_rt]=sub_fn_clean2_cpi(cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt);
end;
if pp==2;
	[cpi_rt,cpi_food_rt,cpi_gasoline_rt]=sub_fn_clean2_cpi(cpi_core_rt,cpi_rt,cpi_food_rt,cpi_gasoline_rt);
end;
if pp==3;
	[cpi_rt,cpi_core_rt,cpi_gasoline_rt]=sub_fn_clean2_cpi(cpi_food_rt,cpi_rt,cpi_core_rt,cpi_gasoline_rt);
end;
if pp==4;
	[cpi_rt,cpi_core_rt,cpi_food_rt]=sub_fn_clean2_cpi(cpi_gasoline_rt,cpi_rt,cpi_core_rt,cpi_food_rt);
end;
%
%---------------------------------------------------------------------
%
function z_out=sub_fn_clean1_cpi(z_in);
% Sub-function to remove vintages that only revise old seasonals.
z_out(:,1)=z_in(:,1); % dates
z_out(:,2)=z_in(:,2); % first column with actual data
for i1=3:size(z_in,2);
	if max(find(~isnan(z_in(:,i1))))>max(find(~isnan(z_in(:,i1-1))));
		z_out=[z_out,z_in(:,i1)];
	end;
end;
%
%---------------------------------------------------------------------
%
function [z1,z2,z3]=sub_fn_clean2_cpi(z0,z1_in,z2_in,z3_in);
% Sub-function to line up vintages across series.
d=z0(1,:);
for i1=1:3;
	z=[];
	eval(['zin=z',num2str(i1),'_in;']);
	z(:,1)=zin(:,1);
	count1=2;
	for i2=2:length(d);
		i3=find(d(i2)==zin(1,:));
		z(:,count1)=zin(:,i3);
		count1=count1+1;
	end;
	eval(['z',num2str(i1),'=z;']);
end;