% fn_compute_g_monthly.m
% Function to compute monthly growth rates from one month to the next.
%  g_x=fn_compute_g_monthly(x);
%
% Input:
% x - A matrix of monthly observations, with variables in columns and
%     time in rows; missing observations take on junk_nos.
% 
% Output:
% g_x - A matrix of size(x) with month-over-month growth rates; missing
%       observations take on junk_nos.
%
function [g_x,g_x_dln]=fn_compute_g_monthly(x);
%
global junk_nos;
%
[r,c]=size(x);
g_x=junk_nos*ones(r,c);
g_x_dln=junk_nos*ones(r,c);
for i2=1:c;
	for i1=min(find(x(:,i2)~=junk_nos))+1:max(find(x(:,i2)~=junk_nos));
		if x(i1,i2)~=junk_nos & x(i1-1,i2)~=junk_nos;
			g_x(i1,i2)=100*(x(i1,i2)/x(i1-1,i2)-1);
			g_x_dln(i1,i2)=100*log(x(i1,i2)/x(i1-1,i2));
		end;
	end;
end;