% fn_compute_initial_q.m
% Function to compute initial growth rates for quarterly data.
%  g_x=fn_compute_initial_q(date_q,date_m,x_rt);
%
% The function takes the real-time dataset and computes the initial
% quarterly (SAAR) inflation readings as percent changes.
%
% Inputs:
% date_q - A vector with quarterly dates.
% date_m - A vector with monthly dates.
% x_rt - A matrix of real-time price levels.
% 
% Outputs:
% g_x - A vector with dimensionality date_q of initial inflation readings.
%
function g_x=fn_compute_initial_q(date_q,date_m,x_rt);
%
global junk_nos;
%
g_x=junk_nos*ones(size(date_q));
% loop over incoming data; need to pay attention to the timing
for i1=2:size(x_rt,2);
	i2=max(find(x_rt(:,i1)~=junk_nos));
	i3=find(date_m==x_rt(i2,1));
	y=round(date_m(i3));
	m=round(100*rem(date_m(i3),1));
	if rem(m,3)==0;
		q=y+m/3/10;
		i4=find(date_q==q);
		if sum(x_rt(i2-2-3:i2,i1)==junk_nos)==0;
			g_x(i4)=100*((mean(x_rt(i2-2:i2,i1))/mean(x_rt(i2-2-3:i2-3,i1)))^4-1);
		end;
	end;
end;