% fn_compute_metric_m.m
% Function to compute growth rates for monthly data.
%  [g_x,g_x_yoy]=fn_compute_metric_m(date_m,x_rt);
%
% The function takes the real-time dataset and computes the 
% monthly (non-annualized) inflation readings as percent changes.
%
% Inputs:
% date_m - A vector with monthly dates.
% x_rt - A matrix of real-time price levels.
% h - Number of months ahead to go to look back and compute monthly inflation.
% 
% Outputs:
% g_x - A vector with dimensionality date_m of monthly inflation readings.
% g_x_yoy - A vector with dimensionality date_m of monthly year-over-year inflatino readings.
%
function [g_x,g_x_yoy,g_x_dln,g_x_yoy_dln]=fn_compute_metric_m(date_m,x_rt,h);
%
global junk_nos;
%
g_x=junk_nos*ones(size(date_m));
g_x_yoy=junk_nos*ones(size(date_m));
g_x_dln=junk_nos*ones(size(date_m));
g_x_yoy_dln=junk_nos*ones(size(date_m));
% loop over incoming data
for i1=2:size(x_rt,2);
	i2=max(find(x_rt(:,i1)~=junk_nos));
	i3=find(date_m==x_rt(i2,1));
	i4=min(i2+h,size(x_rt,1));
	i5=min(find(x_rt(i4,2:end)~=junk_nos));
	if x_rt(i2,i5+1)~=junk_nos & x_rt(i2-1,i5+1)~=junk_nos;
		g_x(i3)=100*(x_rt(i2,i5+1)/x_rt(i2-1,i5+1)-1);
		g_x_dln(i3)=100*log(x_rt(i2,i5+1)/x_rt(i2-1,i5+1));
	end;
	if x_rt(i2,i5+1)~=junk_nos & x_rt(i2-12,i5+1)~=junk_nos;
		g_x_yoy(i3)=100*(x_rt(i2,i5+1)/x_rt(i2-12,i5+1)-1);
		g_x_yoy_dln(i3)=100*log(x_rt(i2,i5+1)/x_rt(i2-12,i5+1));
	end;
end;