% fn_make_forecast_headline.m
% Function to make forecasts for total inflation.
%  f_g_p=fn_make_forecast_headline(f_g_p,g_p_gasoline_m_sa,f_g_p_core,f_g_p_food);
%
% Input:
% f_g_p - Vector of monthly total inflation (potentially includes
%					conversion of total CPI to total PCE).
% g_p_gasoline_m_sa - Vector of monthly gasoline price movements, seasonally
%                     adjusted.
% f_g_p_core - Vector of monthly core inflation, including forecasts
%              for missing data.
% f_g_p_food - Vector of monthly inflation for food, 
%              including forecasts for missing data.
%
% Output: 
% f_g_p - Vector of monthly total inflation, with conversion of 
%         available gasoline price inflation and forecasts of future
%         missing months.
%
function [f_g_p]=fn_make_forecast_headline(f_g_p,g_p_gasoline_m_sa,f_g_p_core,f_g_p_food);
%
global junk_nos tau;
%
f1=max(find(f_g_p~=junk_nos));
L=length(f_g_p);
b=regress(f_g_p(f1-tau+1:f1),[ones(tau,1),f_g_p_core(f1-tau+1:f1),...
	g_p_gasoline_m_sa(f1-tau+1:f1),f_g_p_food(f1-tau+1:f1)]);
for i1=f1+1:L;
	% have gasoline data for that month
	if g_p_gasoline_m_sa(i1)~=junk_nos;
		f_g_p(i1)=b'*[1;f_g_p_core(i1);g_p_gasoline_m_sa(i1);f_g_p_food(i1)];
	else;
	% don't have gasoline data for that and future months
		f_g_p=fn_make_forecast_ma(f_g_p);
		break;
	end;
end;