% fn_make_nowcasts_drop_oil.m
% Function to make inflation nowcasts.
% Modified to drop oil prices.
%  [f_g_cpi,f_g_cpi_core,f_g_pcepm,f_g_pcepm_core]=fn_make_nowcasts_drop_oil(date_m,date_d,date_w_gas,...
%   p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);
%
function [f_g_cpi,f_g_cpi_core,f_g_pcepm,f_g_pcepm_core]=fn_make_nowcasts_drop_oil(date_m,date_d,date_w_gas,...
	p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);
%
global junk_nos tau tau_L J_ sa_years extra_months;
%
% Temporarily assign empties
f_g_cpi=[];
f_g_cpi_core=[];
f_g_pcepm=[];
f_g_pcepm_core=[];
%
% Switches for whether to do PCE inflation or not
if nargin<10;
	pce_switch=0;
	g_cpi_food_at_home=[];
	g_pcepm=[];
	g_pcepm_core=[];
	g_pcepm_goods_food=[];
end;
if nargin==13;
	pce_switch=1;
end;
%
%---------------------------------------------------------------------
% Weekly data
%---------------------------------------------------------------------
a=p_gasoline_w;
aa=zeros(size(a,1),3);
for i1=1:size(a,1);
	aa(i1,1)=floor(date_w_gas(i1)/10000);
	aa(i1,2)=floor((date_w_gas(i1)-10000*aa(i1,1))/100);
	aa(i1,3)=round(date_w_gas(i1)-10000*aa(i1,1)-100*aa(i1,2));
end;
start_date=aa(1,1)+aa(1,2)/100;
end_date=aa(size(aa,1),1)+aa(size(aa,1),2)/100;
f1=find(date_m==start_date);
f2=find(date_m==end_date);
data_names1=char('p_gasoline_m');
a_m=junk_nos*ones(length(date_m),size(data_names1,1));
g_a_m=junk_nos*ones(length(date_m),size(data_names1,1));
for i1=f1:f2;
	dy=round(date_m(i1));
	dm=round(100*date_m(i1))-100*round(date_m(i1));
	f3=find(aa(:,1)==dy & aa(:,2)==dm);
	for i2=1:size(data_names1,1);
		temp1=a(f3,i2);
		temp1=temp1(temp1~=junk_nos);
		if length(temp1)>0;
			a_m(i1,i2)=mean(temp1);
			if a_m(i1-1,i2)~=junk_nos;
				g_a_m(i1,i2)=100*(a_m(i1,i2)/a_m(i1-1,i2)-1);
			end;
		end;
	end;
end;
for i1=1:size(data_names1,1);
	eval([data_names1(i1,:),'=a_m(:,',num2str(i1),');']);
	eval(['g_',data_names1(i1,:),'=g_a_m(:,',num2str(i1),');']);
end;
%
%---------------------------------------------------------------------
% Daily data
%---------------------------------------------------------------------
%%a=p_oil_d;
%%aa=zeros(size(a,1),3);
%%for i1=1:size(a,1);
%%	aa(i1,1)=floor(date_d(i1)/10000);
%%	aa(i1,2)=floor((date_d(i1)-10000*aa(i1,1))/100);
%%	aa(i1,3)=round(date_d(i1)-10000*aa(i1,1)-100*aa(i1,2));
%%end;
%%start_date=aa(1,1)+aa(1,2)/100;
%%end_date=aa(size(aa,1),1)+aa(size(aa,1),2)/100;
%%f1=find(date_m==start_date);
%%f2=find(date_m==end_date);
%%p_oil_m=junk_nos*ones(length(date_m),1);
%%for i1=f1:f2;
%%	dy=round(date_m(i1));
%%	dm=round(100*date_m(i1))-100*round(date_m(i1));
%%	f3=find(aa(:,1)==dy & aa(:,2)==dm);
%%	temp1=p_oil_d(f3);
%%	temp1=temp1(temp1~=junk_nos);
%%	if isempty(temp1)==0;
%%		p_oil_m(i1)=mean(temp1);
%%	end;
%%end;
%
%---------------------------------------------------------------------
% Use oil prices to forecast gasoline prices
%---------------------------------------------------------------------
%%% use random walk forecast for crude oil prices
%%f_p_oil_m=p_oil_m;
%%f1=max(find(p_oil_d~=junk_nos)); % use last daily observation
%%last_p_oil_d=p_oil_d(f1);
%%f1=max(find(p_oil_m~=junk_nos));
%%% add extra month(s)
%%f_p_oil_m(f1+1:f1+extra_months)=last_p_oil_d;
% seasonally adjust gasoline price movements
g_p_gasoline_m_sa=junk_nos*ones(size(date_m));
g_p_gasoline_errors=g_p_gasoline_m_sa;
s1=min(find(g_p_gasoline_m~=junk_nos));
e1=max(find(g_p_gasoline_m~=junk_nos));
for i1=s1:e1;
	if g_cpi_gasoline(i1)~=junk_nos;
		g_p_gasoline_errors(i1)=g_p_gasoline_m(i1)-g_cpi_gasoline(i1);
	end;
end;
for i1=s1+sa_years*12:e1;
	g_p_gasoline_m_sa(i1)=g_p_gasoline_m(i1)-mean(g_p_gasoline_errors([i1-12*[1:sa_years]]));
end;
% form forecasts for gasoline prices; use an error correction model, in which
% 1. p_gasoline_star(t)=a+b*p_oil_m(t)
% 2. p_gasoline_m(t)-p_gasoline_m(t-1)=c*(p_oil_m(t)-p_oil_m(t-1))+d*(p_gasoline_m(t-1)-p_gasoline_star(t-1))
% Note: if assume that c=b, then 2 can be written in gap form as 
% 2'. p_gasoline_m(t)-p_gasoline_star(t)=d*(p_gasoline_m(t-1)-p_gasoline_star(t-1))
f_p_gasoline_m=p_gasoline_m;
f_g_p_gasoline_m=g_p_gasoline_m;
%%f1=max(find(p_gasoline_m~=junk_nos));
%%f2=max(f1-tau_L+1,min(find(p_gasoline_m~=junk_nos)));
%%% estimate the first stage regression 
%%o=ones(f1-f2+1,1);
%%b=regress(p_gasoline_m(f2:f1),[o,f_p_oil_m(f2:f1)]);
%%p_gasoline_star=junk_nos*ones(size(p_gasoline_m));
%%p_gasoline_star(f2:f1)=[o,f_p_oil_m(f2:f1)]*b;
%%% estimate the second stage regression
%%% set c=b to provide a gap interpretation (modified error correction model)
%%bb=regress(p_gasoline_m(f2+1:f1)-p_gasoline_m(f2:f1-1)-b(2)*(f_p_oil_m(f2+1:f1)-f_p_oil_m(f2:f1-1)),p_gasoline_m(f2:f1-1)-p_gasoline_star(f2:f1-1));
%%for i1=f1+1:max(find(f_p_oil_m~=junk_nos));
%%	p_gasoline_star(i1)=b'*[1;f_p_oil_m(i1)];
%%	f_p_gasoline_m(i1)=f_p_gasoline_m(i1-1)+b(2)*(f_p_oil_m(i1)-f_p_oil_m(i1-1))+bb*(f_p_gasoline_m(i1-1)-p_gasoline_star(i1-1));
%%	f_g_p_gasoline_m(i1)=100*(f_p_gasoline_m(i1)/f_p_gasoline_m(i1-1)-1);
%%end;
% seasonally adjust forecasts
f_g_p_gasoline_m_sa=g_p_gasoline_m_sa;
for i1=f1+1:max(find(f_g_p_gasoline_m~=junk_nos));
	f_g_p_gasoline_m_sa(i1)=f_g_p_gasoline_m(i1)-mean(g_p_gasoline_errors([i1-12*[1:sa_years]]));
end;
%
%---------------------------------------------------------------------
%---------------------------------------------------------------------
% Nowcast inflation
%---------------------------------------------------------------------
%---------------------------------------------------------------------
% monthly price levels/inflation rates
f_g_cpi=g_cpi;
f_g_cpi_core=g_cpi_core;
if pce_switch;
f_g_pcepm=g_pcepm;
f_g_pcepm_core=g_pcepm_core;
f_g_pcepm_goods_food=g_pcepm_goods_food;
end; % pce_switch
%
% In general, for a missing month will have:
% (1) nothing
% (2) oil and gasoline prices/forecasts
% (3) oil and gasoline prices/forecasts and CPI
% For case 1, make own forecasts for core and headline inflation.
% For case 2, make a forecast of core and food prices, then combine with gasoline to derive headline.
% For case 3, convert CPI to PCEPI for one month; beyond that, in case 2 or 1.
%
%-------------------------
% Start with CPI inflation
%-------------------------
%
%-------------------------
% Core
%
f_g_cpi_core=fn_make_forecast_ma(g_cpi_core);
%
%-------------------------
% Headline
%
% First forecast food
f_g_cpi_food=fn_make_forecast_ma(g_cpi_food);
% Combine core forecast and food forecast with available gasoline prices/forecasts
f_g_cpi=fn_make_forecast_headline(g_cpi,f_g_p_gasoline_m_sa,f_g_cpi_core,f_g_cpi_food);
%
%----------------------
% Next do PCE inflation
%----------------------
if pce_switch;
%
% Check whether there is more CPI data than PCE data
f1=max(find(g_pcepm~=junk_nos));
f1a=max(find(g_pcepm_goods_food~=junk_nos));
f2=max(find(g_cpi~=junk_nos));
f2a=max(find(g_cpi_food_at_home~=junk_nos));
%
%----------------------
% Core
%
% Case: have one extra CPI reading
if f2>f1;
	% Convert additional core CPI reading to core PCE
	b_core_inf=regress(g_pcepm_core(f1-tau+1:f1),[ones(tau,1),g_cpi_core(f1-tau+1:f1)]);
	extra_mos=f2-f1;
	for i1=1:extra_mos;
		f_g_pcepm_core(f1+i1)=b_core_inf'*[1;g_cpi_core(f1+i1)];
	end;
end;
%
f_g_pcepm_core=fn_make_forecast_ma(f_g_pcepm_core);
%
%----------------------
% Headline
%
% First forecast food
% Case: have one extra CPI reading
if f2a>f1a;
	extra_mos=f2a-f1a;
	for i1=1:extra_mos;
		f_g_pcepm_goods_food(f1a+i1)=g_cpi_food_at_home(f1a+i1);
	end;
end;
f_g_pcepm_goods_food=fn_make_forecast_ma(f_g_pcepm_goods_food);
%
% Case: have one extra CPI reading
if f2>f1;
	% Convert additional headline CPI reading to headline PCE
	b_total_inf=regress(g_pcepm(f1-tau+1:f1),[ones(tau,1),g_cpi(f1-tau+1:f1)]);
	extra_mos=f2-f1;
	for i1=1:extra_mos;
		f_g_pcepm(f1+i1)=b_total_inf'*[1;g_cpi(f1+i1)];
	end;
end;
%
% Combine core forecast and food forecast with available gasoline prices/forecasts
f_g_pcepm=fn_make_forecast_headline(f_g_pcepm,f_g_p_gasoline_m_sa,f_g_pcepm_core,f_g_pcepm_goods_food);
%
end; % pce_switch