% fn_spf_to_standard_matrices.m
% Function to convert the SPF forecasts from raw form to the standardized 
% real-time form used for the other variables.
%  [spf_cpi_raw,spf_cpi_core_raw,spf_pcep_raw,spf_pcep_core_raw]=...
%   fn_spf_to_standard_matrices(txt);
%
% The function loads the unformatted SPF data along with the dates that
% the forecasts were made and then puts them into a standardized
% format, for the sake of comparison with the other data sources.
%
% Input:
% txt - A text string indicating either 'median' or 'mean'.
% 
% Output:
% spf_cpi_rt - A matrix with the SPF CPI forecasts, either using
%              medians or means (per user specification).
% spf_cpi_core_rt - A matrix with the SPF core CPI forecasts.
% spf_pcep_rt - A matrix with the SPF PCE forecasts.
% spf_pcep_core_rt - A matrix with the SPF core PCE forecasts.
%
function [spf_cpi_rt,spf_cpi_core_rt,spf_pcep_rt,spf_pcep_core_rt]=...
	fn_spf_to_standard_matrices(txt);
%
% load data from Philly Fed
txt1=['spf_',txt,'_cpi'];
spf_cpi=xlsread('nowcast_inflation_data',txt1);
txt1=['spf_',txt,'_cpi_core'];
spf_cpi_core=xlsread('nowcast_inflation_data',txt1);
txt1=['spf_',txt,'_pcep'];
spf_pcep=xlsread('nowcast_inflation_data',txt1);
txt1=['spf_',txt,'_pcep_core'];
spf_pcep_core=xlsread('nowcast_inflation_data',txt1);
% load dates/deadlines for survey responses
spf_dates=xlsread('nowcast_inflation_data','spf_dates');
%
spf_rt=NaN*ones(size(spf_dates,1)+4+1,size(spf_dates,1)+1);
% set up dates (rows)
date_q=spf_dates(:,1);
date_q=[date_q;date_q(length(date_q)-3:length(date_q))+1];
spf_rt(2:size(spf_rt,1),1)=date_q;
%
% loop over cases
%loop_no=1;
for loop_no=1:4;
%
if loop_no==1; txt1='cpi'; end;
if loop_no==2; txt1='cpi_core'; end;
if loop_no==3; txt1='pcep'; end;
if loop_no==4; txt1='pcep_core'; end;
%
spf_temp=spf_rt;
eval(['frbphil_data=spf_',txt1,';']);
for i1=1:size(spf_dates,1);
	% put in survey deadline in top row
	spf_temp(1,i1+1)=spf_dates(i1,2);
	% find matching row forecasts from FRB-Phil
	i2=find(spf_dates(i1,1)==frbphil_data(:,1)+frbphil_data(:,2)/10);
	% put into the right spots
%	spf_temp(i1+1:i1+4+1,i1+1)=frbphil_data(i2,4:8)';
	spf_temp(i1+1-1:i1+4+1,i1+1)=frbphil_data(i2,3:8)';
end;
i1=spf_temp==-999;
spf_temp(i1)=NaN;
eval(['spf_',txt1,'_rt=spf_temp;']);
%
end;