% nowcasting_inflation_monthly.m
% Program to produce monthly results.

% Load the data
nowcasting_inflation_setup;

%% Put in preliminaries here for MIDAS and dynamic factor model		-- here and elsewhere: comment out components from MIDAS and DFM to not reshare code but let the code run without incident 
%RootDir = cd;
%FunDir  = [RootDir,'/funcs/'];
%addpath(genpath(FunDir));

% Set metrics to assess performance
g_cpi_metrics_m_yoy=[g_cpi_metric,g_cpi_nsa_metric_yoy]; % note year-over-year metric is based on NSA data, following the BLS
g_cpi_core_metrics_m_yoy=[g_cpi_core_metric,g_cpi_core_nsa_metric_yoy]; % note year-over-year metric is based on NSA data, following the BLS
g_pcepm_metrics_m_yoy=[g_pcepm_metric,g_pcepm_metric_yoy];
g_pcepm_core_metrics_m_yoy=[g_pcepm_core_metric,g_pcepm_core_metric_yoy];

%---------------------------------------------------------------------
disp('-------------------------------------------------------------------------------');
disp('Monthly nowcasting exercise');
disp('-------------------------------------------------------------------------------');
%---------------------------------------------------------------------
% Using monthly data, compute month-over-month inflation rates and 
% year-over-year inflation rates; the latter are featured in Modugno (2013) 
% and Monteforte and Moretti (2013).
% 
% For the monthly nowcasting exercise, there are 7 key dates:
% 1. Day before the month begins--have no data for the current month, 
% 	 but assume you have CPI and PCE for two months earlier.
% 2. Day #8 of month--have one week of data (oil and gasoline prices).
% 3. Day #15 of the month--assume that CPI for the previous month is 
% 	 released no later than this day.
% 4. Day #22 of month--have three weeks of data (oil and gasoline
%    prices) and previous month's CPI
% 5. Final day of the month--assume that PCE for the previous month is
% 	 released no later than this day.
% 6. Middle of next month--receive CPI for the month being nowcasted
%   (for PCE forecasting case).
% 7. End of next month--receive PCE for the month being nowcasted (note:
%    this won't be part of the nowcast set)
no_cases_m=6;

disp(['Month to start the analysis: ',num2str(start_month)]);
disp(['Month to end the analysis: ',num2str(end_month)]);

% Monthly statistical models
% Alternative cases to compare against the model
% 1. Random walk
% set matrices to store alternative nowcasts
no_alt_cases1=1;

% Other monthly nowcasting models
% 1. Small MIDAS model
% 2. Dynamic factor model (Modugno)
no_alt_cases2=2;

no_alt_cases=no_alt_cases1+no_alt_cases2;

% Set matrices to store inflation nowcasts
% Dimension 1 is time
% Dimension 2 is cases
% Dimension 3 is the model; =1 is baseline, =2 is random walk, =3 is MIDAS, =4 is dynamic factor model
% Dimension 4 is the inflation measure; =1 for month-over-month and =2 for year-over-year
%g_cpi_nowcasts=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_cpi_core_nowcasts=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_pcepm_nowcasts=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_pcepm_core_nowcasts=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_cpi_nowcasts_last=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_cpi_core_nowcasts_last=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_pcepm_nowcasts_last=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
%g_pcepm_core_nowcasts_last=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
% Load results from MIDAS model and DFM; clear baseline and random walk
load monthly_nowcasts;
g_cpi_nowcasts(:,:,1:2,:)=junk_nos;
g_cpi_nowcasts_last(:,:,1:2,:)=junk_nos;
g_cpi_core_nowcasts(:,:,1:2,:)=junk_nos;
g_cpi_core_nowcasts_last(:,:,1:2,:)=junk_nos;
g_pcepm_nowcasts(:,:,1:2,:)=junk_nos;
g_pcepm_nowcasts_last(:,:,1:2,:)=junk_nos;
g_pcepm_core_nowcasts(:,:,1:2,:)=junk_nos;
g_pcepm_core_nowcasts_last(:,:,1:2,:)=junk_nos;

% Directional forecast accuracy matrices
dfa_g_cpi_forecast_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_cpi_data_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_cpi_core_forecast_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_cpi_core_data_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_pcepm_forecast_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_pcepm_data_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_pcepm_core_forecast_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);
dfa_g_pcepm_core_data_direction=junk_nos*ones(length(junk_nos_m),no_cases_m,1+no_alt_cases,2);

%---------------------------------------------------------------------
for mcount=find(date_m==start_month):find(date_m==end_month);
%---------------------------------------------------------------------

yyyymm=date_m(mcount); % date of month of interest
yy=round(yyyymm);
mm=round(100*yyyymm-100*round(yyyymm));
ff=mcount; % position of the month of interest

disp(['Month = ',num2str(date_m(mcount))]);

%---------------------------------------------------------------------
for case_count=1:no_cases_m;
%---------------------------------------------------------------------

%---------------------------------------------------------------------
% Case 1: 
% right before beginning of month, 
% no gasoline or oil price data, 
% missing 2 months of CPI data,
% missing 2 months of PCE data
%---------------------------------------------------------------------
if case_count==1;
date_cutoff=yy*10000+(mm-1)*100+31;
ff_cpi=ff-2; % missing 2 months
ff_pcepm=ff-2; % missing 2 months
end;

%---------------------------------------------------------------------
% Case 2: 
% day 8 of the month, 
% some gasoline and oil price data, 
% missing 2 months of CPI data,
% missing 2 months of PCE data
%---------------------------------------------------------------------
if case_count==2;
date_cutoff=yy*10000+(mm)*100+08;
ff_cpi=ff-2; % missing 2 months
ff_pcepm=ff-2; % missing 2 months
end;

%---------------------------------------------------------------------
% Case 3: 
% day 15 of the month, 
% some gasoline and oil price data, 
% assume get CPI for previous month on 15th (missing 1 month),
% missing 2 months of PCE data
%---------------------------------------------------------------------
if case_count==3;
date_cutoff=yy*10000+(mm)*100+15;
ff_cpi=ff-1; % missing 1 month
ff_pcepm=ff-2; % missing 2 months
end;

%---------------------------------------------------------------------
% Case 4: 
% day 22 of the month, 
% some gasoline and oil price data, 
% missing 1 month of CPI data,
% missing 2 months of PCE data
%---------------------------------------------------------------------
if case_count==4;
date_cutoff=yy*10000+(mm)*100+22;
ff_cpi=ff-1; % missing 1 month
ff_pcepm=ff-2; % missing 2 months
end;

%---------------------------------------------------------------------
% Case 5: 
% last day of the month, 
% all gasoline and oil price data, 
% missing 1 month of CPI data,
% assume get PCE for previous month on last day (missing 1 month)
%---------------------------------------------------------------------
if case_count==5;
date_cutoff=yy*10000+(mm+1)*100+00;
ff_cpi=ff-1; % missing 1 month
ff_pcepm=ff-1; % missing 1 month
end;

%---------------------------------------------------------------------
% Case 6: 
% middle of the following month, 
% all gasoline and oil price data, 
% assume get CPI for previous month on 15th (missing 0 months),
% missing 1 month of PCE data
%---------------------------------------------------------------------
if case_count==6;
date_cutoff=yy*10000+(mm+1)*100+15;
ff_cpi=ff; % missing 0 months
ff_pcepm=ff-1; % missing 1 month
end;

%---------------------------------------------------------------------
% Set up available data
%---------------------------------------------------------------------
wcount=max(find(date_w<=date_cutoff));
dcount=max(find(date_d<=date_cutoff));
f1=min(find(cpi_rt(ff_cpi+1,2:end)~=junk_nos));
mcount_cpi=1+f1;
if pce_switch;
f1=min(find(pcepm_rt(ff_pcepm+1,2:end)~=junk_nos));
mcount_pcepm=1+f1;
end;
%
p_oil_d=junk_nos*ones(size(date_d));
p_oil_d(1:dcount)=p_oil_d_rt(1:dcount,2);
%p_oil_wti_d=junk_nos*ones(size(date_d));
%p_oil_wti_d(1:dcount)=p_oil_wti_d_rt(1:dcount,2);
%p_foodstuffs_d=junk_nos*ones(size(date_d));
%p_foodstuffs_d(1:dcount)=p_foodstuffs_d_rt(1:dcount,2);
%p_all_commodities_d=junk_nos*ones(size(date_d,1),size(p_all_commodities_d_rt,2)-1);
%p_all_commodities_d(1:dcount,:)=p_all_commodities_d_rt(1:dcount,2:end);
%
p_gasoline_w=junk_nos*ones(size(date_w));
p_gasoline_w(1:wcount)=p_gasoline_w_rt(1:wcount,2);
%p_gasoline_by_grade_w=junk_nos*ones(size(date_w,1),size(p_gasoline_by_grade_w_rt,2)-1);
%p_gasoline_by_grade_w(1:wcount,:)=p_gasoline_by_grade_w_rt(1:wcount,2:end);
%
cpi=cpi_rt(2:end,mcount_cpi);
[g_cpi,g_cpi_dln]=fn_compute_g_monthly(cpi);
cpi_core=cpi_core_rt(2:end,mcount_cpi);
[g_cpi_core,g_cpi_core_dln]=fn_compute_g_monthly(cpi_core);
cpi_food=cpi_food_rt(2:end,mcount_cpi);
[g_cpi_food,g_cpi_food_dln]=fn_compute_g_monthly(cpi_food);
cpi_gasoline=cpi_gasoline_rt(2:end,mcount_cpi);
[g_cpi_gasoline,g_cpi_gasoline_dln]=fn_compute_g_monthly(cpi_gasoline);
% unfortunately, don't have real-time CPI--food at home (which 
% conceptually matches up with PCE--goods--food), use CPI--food instead
g_cpi_food_at_home=g_cpi_food;
g_cpi_food_at_home_dln=g_cpi_food_dln;
%
if pce_switch;
pcepm=pcepm_rt(2:end,mcount_pcepm);
[g_pcepm,g_pcepm_dln]=fn_compute_g_monthly(pcepm);
pcepm_core=pcepm_core_rt(2:end,mcount_pcepm);
[g_pcepm_core,g_pcepm_core_dln]=fn_compute_g_monthly(pcepm_core);
% unfortunately, don't have real-time PCE--goods--food, use CPI--food instead
g_pcepm_goods_food=g_cpi_food;
g_pcepm_goods_food_dln=g_cpi_food_dln;
end; % pce_switch
%
% NSA data for year-over-year exercises
i1=cpi~=junk_nos;
cpi_nsa=junk_nos_m;
cpi_core_nsa=junk_nos_m;
cpi_nsa(i1)=cpi_nsa_rt(i1);
cpi_core_nsa(i1)=cpi_core_nsa_rt(i1);


%---------------------------------------------------------------------
% Make nowcasts
%---------------------------------------------------------------------
[f_g_cpi,f_g_cpi_core,f_g_pcepm,f_g_pcepm_core]=fn_make_nowcasts(date_m,date_d,date_w,...
	p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	

%---------------------------
% Baseline model position =1
%---------------------------
model_counter=1;

%------------------
% Save the nowcasts
%------------------
f_cpi=cpi;
f_cpi_core=cpi_core;
f2=max(find(cpi~=junk_nos));
for i1=f2+1:ff;
	f_cpi(i1)=f_cpi(i1-1)*(1+f_g_cpi(i1)/100);
	f_cpi_core(i1)=f_cpi_core(i1-1)*(1+f_g_cpi_core(i1)/100);
end;
g_cpi_nowcasts(mcount,case_count,model_counter,1)=f_g_cpi(ff); % month-over-month
g_cpi_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi(ff)/f_cpi(ff-12)-1); % year-over-year; note: compute using SA data, but the metric will be the truth using NSA data as reported by the BLS
g_cpi_nowcasts_last(mcount,case_count,model_counter,1)=f_g_cpi(ff-1);
g_cpi_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi(ff-1)/f_cpi(ff-1-12)-1);
g_cpi_core_nowcasts(mcount,case_count,model_counter,1)=f_g_cpi_core(ff); % month-over-month
g_cpi_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff)/f_cpi_core(ff-12)-1); % year-over-year; note: compute using SA data, but the metric will be the truth using NSA data as reported by the BLS
g_cpi_core_nowcasts_last(mcount,case_count,model_counter,1)=f_g_cpi_core(ff-1);
g_cpi_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-12)-1);
if pce_switch;
f_pcepm=pcepm;
f_pcepm_core=pcepm_core;
f1=max(find(pcepm~=junk_nos));
for i1=f1+1:ff;
	f_pcepm(i1)=f_pcepm(i1-1)*(1+f_g_pcepm(i1)/100);
	f_pcepm_core(i1)=f_pcepm_core(i1-1)*(1+f_g_pcepm_core(i1)/100);
end;
g_pcepm_nowcasts(mcount,case_count,model_counter,1)=f_g_pcepm(ff); % month-over-month
g_pcepm_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm(ff)/f_pcepm(ff-12)-1); % year-over-year
g_pcepm_nowcasts_last(mcount,case_count,model_counter,1)=f_g_pcepm(ff-1); 
g_pcepm_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-12)-1);
g_pcepm_core_nowcasts(mcount,case_count,model_counter,1)=f_g_pcepm_core(ff); % month-over-month
g_pcepm_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff)/f_pcepm_core(ff-12)-1); % year-over-year
g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,1)=f_g_pcepm_core(ff-1); % month-over-month
g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-12)-1); % year-over-year
end; % pce_switch


%---------------------------------------------------------------------
% Alternative forecasts
%---------------------------------------------------------------------


%---------------------------------------------------------------------
% Random walk baseline
%---------------------------------------------------------------------
model_counter=2;
%
% Month-over-month: just use most recent monthly reading
f2=max(find(g_cpi~=junk_nos));
f_g_cpi=g_cpi;
f_g_cpi_core=g_cpi_core;
f_g_cpi(f2+1:ff)=g_cpi(f2);
f_g_cpi_core(f2+1:ff)=g_cpi_core(f2);
g_cpi_nowcasts(mcount,case_count,model_counter,1)=f_g_cpi(ff);
g_cpi_nowcasts_last(mcount,case_count,model_counter,1)=f_g_cpi(ff-1);
g_cpi_core_nowcasts(mcount,case_count,model_counter,1)=f_g_cpi_core(ff);
g_cpi_core_nowcasts_last(mcount,case_count,model_counter,1)=f_g_cpi_core(ff-1);
if pce_switch;
f1=max(find(g_pcepm~=junk_nos));
f_g_pcepm=g_pcepm;
f_g_pcepm_core=g_pcepm_core;
f_g_pcepm(f1+1:ff)=g_pcepm(f1);
f_g_pcepm_core(f1+1:ff)=g_pcepm_core(f1);
g_pcepm_nowcasts(mcount,case_count,model_counter,1)=f_g_pcepm(ff);
g_pcepm_nowcasts_last(mcount,case_count,model_counter,1)=f_g_pcepm(ff-1);
g_pcepm_core_nowcasts(mcount,case_count,model_counter,1)=f_g_pcepm_core(ff);
g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,1)=f_g_pcepm_core(ff-1);
end;
%
% Year-over-year: use most recent year-over-year reading
f2=max(find(g_cpi~=junk_nos));
g_cpi_nowcasts(mcount,case_count,model_counter,2)=100*(cpi(f2)/cpi(f2-12)-1);
g_cpi_core_nowcasts(mcount,case_count,model_counter,2)=100*(cpi_core(f2)/cpi_core(f2-12)-1);
f_cpi=cpi;
f_cpi_core=cpi_core;
for i1=f2+1:ff;
	f_cpi(i1)=f_cpi(i1-12)*(1+g_cpi_nowcasts(mcount,case_count,model_counter,2)/100);
	f_cpi_core(i1)=f_cpi_core(i1-12)*(1+g_cpi_core_nowcasts(mcount,case_count,model_counter,2)/100);
end;
g_cpi_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi(ff-1)/f_cpi(ff-1-12)-1);
g_cpi_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-12)-1);
if pce_switch;
f1=max(find(g_pcepm~=junk_nos));
g_pcepm_nowcasts(mcount,case_count,model_counter,2)=100*(pcepm(f1)/pcepm(f1-12)-1);
g_pcepm_core_nowcasts(mcount,case_count,model_counter,2)=100*(pcepm_core(f1)/pcepm_core(f1-12)-1);
f_pcepm=pcepm;
f_pcepm_core=pcepm_core;
for i1=f1+1:ff;
	f_pcepm(i1)=f_pcepm(i1-12)*(1+g_pcepm_nowcasts(mcount,case_count,model_counter,2)/100);
	f_pcepm_core(i1)=f_pcepm_core(i1-12)*(1+g_pcepm_core_nowcasts(mcount,case_count,model_counter,2)/100);
end;
g_pcepm_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-12)-1);
g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-12)-1);
end;


%---------------------------------------------------------------------
% Make other nowcasts from competing monthly models
%---------------------------------------------------------------------


%%--------------------------------------
%% Other nowcasting model: MIDAS (small)
%%--------------------------------------
%model_counter=3;
%%
%% make the nowcasts
%% using natural log monthly growth rates inside the model and as outputs; need to convert
%%
%% CPI
%daily_data_factor=[date_d,p_oil_d];
%daily_series={'crudeoil'};
%daily_transform=ones(1,1);
%weekly_data_factor=[date_w,p_gasoline_w];
%weekly_series={'reggrade'};
%weekly_transform=ones(1,1);
%monthly_data_factor=[date_m,g_cpi_dln,g_cpi_core_dln,g_cpi_gasoline_dln,g_cpi_food_dln];
%monthly_series={'cpi','cpi_core','cpi_gasoline','cpi_food'};
%monthly_transform=zeros(4,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1;
%no_months_to_do=ff-max(find(g_cpi~=junk_nos));
%[f_g_cpi_dln]=fn_make_nowcasts_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);
%%
%% Core CPI
%monthly_data_factor=[date_m,g_cpi_core_dln,g_cpi_dln];
%monthly_series={'cpi_core','cpi'};
%monthly_transform=zeros(2,1);
%midas_lags=1;
%use_Daily=1;
%use_Weekly=1;
%no_months_to_do=ff-max(find(g_cpi_core~=junk_nos));
%[f_g_cpi_core_dln]=fn_make_nowcasts_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);
%%
%% PCE
%monthly_data_factor=[date_m,g_pcepm_dln,g_cpi_dln,g_pcepm_core_dln,g_cpi_gasoline_dln,g_cpi_food_dln];
%monthly_series={'pce','cpi','core_pce','cpi_gasoline','cpi_food'};
%monthly_transform=zeros(5,1);
%midas_lags=1;
%use_Daily=1;
%use_Weekly=1;
%no_months_to_do=ff-max(find(g_pcepm~=junk_nos));
%[f_g_pcepm_dln]=fn_make_nowcasts_PCE_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);	
%%
%% Core PCE
%monthly_data_factor=[date_m,g_pcepm_core_dln,g_cpi_core_dln];
%monthly_series={'core_pce','core_cpi'};
%monthly_transform=zeros(2,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1;
%no_months_to_do=ff-max(find(g_pcepm_core~=junk_nos));
%[f_g_pcepm_core_dln]=fn_make_nowcasts_PCE_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);     
%%
%% Convert growth rates to levels
%f_cpi=cpi;
%f_cpi_core=cpi_core;
%f2=max(find(cpi~=junk_nos));
%for i1=f2+1:ff;
%	f_cpi(i1)=exp(log(f_cpi(i1-1))+f_g_cpi_dln(i1)/100);
%	f_cpi_core(i1)=exp(log(f_cpi_core(i1-1))+f_g_cpi_core_dln(i1)/100);
%end;
%if pce_switch;
%f_pcepm=pcepm;
%f_pcepm_core=pcepm_core;
%f1=max(find(pcepm~=junk_nos));
%for i1=f1+1:ff;
%	f_pcepm(i1)=exp(log(f_pcepm(i1-1))+f_g_pcepm_dln(i1)/100);
%	f_pcepm_core(i1)=exp(log(f_pcepm_core(i1-1))+f_g_pcepm_core_dln(i1)/100);
%end;
%end; % pce_switch
%%
%% Save the nowcasts
%% month-over-month
%g_cpi_nowcasts(mcount,case_count,model_counter,1)=100*(f_cpi(ff)/f_cpi(ff-1)-1);
%g_cpi_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_cpi(ff-1)/f_cpi(ff-1-1)-1);
%g_cpi_core_nowcasts(mcount,case_count,model_counter,1)=100*(f_cpi_core(ff)/f_cpi_core(ff-1)-1);
%g_cpi_core_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-1)-1);
%if pce_switch;
%g_pcepm_nowcasts(mcount,case_count,model_counter,1)=100*(f_pcepm(ff)/f_pcepm(ff-1)-1);
%g_pcepm_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-1)-1);
%g_pcepm_core_nowcasts(mcount,case_count,model_counter,1)=100*(f_pcepm_core(ff)/f_pcepm_core(ff-1)-1);
%g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-1)-1);
%end; % pce_switch
%% year-over-year
%g_cpi_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi(ff)/f_cpi(ff-12)-1);
%g_cpi_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi(ff-1)/f_cpi(ff-1-12)-1);
%g_cpi_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff)/f_cpi_core(ff-12)-1);
%g_cpi_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-12)-1);
%if pce_switch;
%g_pcepm_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm(ff)/f_pcepm(ff-12)-1);
%g_pcepm_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-12)-1);
%g_pcepm_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff)/f_pcepm_core(ff-12)-1);
%g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-12)-1);
%end; % pce_switch



%%---------------------------------------------------------------------
%% Other nowcasting model: Dynamic factor model
%%---------------------------------------------------------------------
%model_counter=4;
%%
%% make the nowcasts
%% using natural log monthly growth rates inside the model and as outputs; need to convert
%%
%% Large version, similar to Modugno (2013):
%daily_data_factor=[date_d,[p_oil_d,p_all_commodities_d(:,[2:14])]];
%daily_series={'crudeoil','foodstuffs','cereal','foodoils','sugar','rawindustrials','rawag','spinning','metals','steel','treas10yr','treas3mo','sp500','fxmajor'};
%daily_transform=ones(14,1);
%daily_transform(11,1)=2; daily_transform(12,1)=2;
%weekly_data_factor=[date_w,p_gasoline_by_grade_w];
%weekly_series={'reggrade','midgrade','premium','diesel'};
%weekly_transform=ones(4,1); 
%monthly_data_factor=[date_m,g_cpi_dln,g_cpi_core_dln,g_pcepm_dln,g_pcepm_core_dln,g_cpi_food_dln,g_cpi_gasoline_dln];
%monthly_series={'cpi','corecpi','pce','corepce','cpi_food','cpi_gasoline'};
%monthly_transform=zeros(6,1);
%factor_lags=6;
%[f_g_cpi_dln,f_g_cpi_core_dln,f_g_pcepm_dln,f_g_pcepm_core_dln]=fn_make_nowcasts_factormodel(daily_data_factor,weekly_data_factor,monthly_data_factor,factor_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform);
%%
%% Convert growth rates to levels
%f_cpi=cpi;
%f_cpi_core=cpi_core;
%f2=max(find(cpi~=junk_nos));
%for i1=f2+1:ff;
%	f_cpi(i1)=exp(log(f_cpi(i1-1))+f_g_cpi_dln(i1)/100);
%	f_cpi_core(i1)=exp(log(f_cpi_core(i1-1))+f_g_cpi_core_dln(i1)/100);
%end;
%if pce_switch;
%f_pcepm=pcepm;
%f_pcepm_core=pcepm_core;
%f1=max(find(pcepm~=junk_nos));
%for i1=f1+1:ff;
%	f_pcepm(i1)=exp(log(f_pcepm(i1-1))+f_g_pcepm_dln(i1)/100);
%	f_pcepm_core(i1)=exp(log(f_pcepm_core(i1-1))+f_g_pcepm_core_dln(i1)/100);
%end;
%end; % pce_switch
%%
%% Save the nowcasts
%% month-over-month
%g_cpi_nowcasts(mcount,case_count,model_counter,1)=100*(f_cpi(ff)/f_cpi(ff-1)-1);
%g_cpi_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_cpi(ff-1)/f_cpi(ff-1-1)-1);
%g_cpi_core_nowcasts(mcount,case_count,model_counter,1)=100*(f_cpi_core(ff)/f_cpi_core(ff-1)-1);
%g_cpi_core_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-1)-1);
%if pce_switch;
%g_pcepm_nowcasts(mcount,case_count,model_counter,1)=100*(f_pcepm(ff)/f_pcepm(ff-1)-1);
%g_pcepm_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-1)-1);
%g_pcepm_core_nowcasts(mcount,case_count,model_counter,1)=100*(f_pcepm_core(ff)/f_pcepm_core(ff-1)-1);
%g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,1)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-1)-1);
%end; % pce_switch
%% year-over-year
%g_cpi_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi(ff)/f_cpi(ff-12)-1);
%g_cpi_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi(ff-1)/f_cpi(ff-1-12)-1);
%g_cpi_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff)/f_cpi_core(ff-12)-1);
%g_cpi_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_cpi_core(ff-1)/f_cpi_core(ff-1-12)-1);
%if pce_switch;
%g_pcepm_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm(ff)/f_pcepm(ff-12)-1);
%g_pcepm_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm(ff-1)/f_pcepm(ff-1-12)-1);
%g_pcepm_core_nowcasts(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff)/f_pcepm_core(ff-12)-1);
%g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,2)=100*(f_pcepm_core(ff-1)/f_pcepm_core(ff-1-12)-1);
%end; % pce_switch


%---------------------------------------------------------------------
% Directional forecast accuracy
%---------------------------------------------------------------------
for model_counter=1:1+no_alt_cases;
for i1=1:2;
%
% CPI, nowcast
if g_cpi_nowcasts(mcount,case_count,model_counter,i1)-g_cpi_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_cpi_forecast_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_cpi_forecast_direction(mcount,case_count,model_counter,i1)=0;
end;
% CPI, data
if g_cpi_metrics_m_yoy(ff,i1)-g_cpi_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_cpi_data_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_cpi_data_direction(mcount,case_count,model_counter,i1)=0;
end;
% Core CPI, nowcast
if g_cpi_core_nowcasts(mcount,case_count,model_counter,i1)-g_cpi_core_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_cpi_core_forecast_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_cpi_core_forecast_direction(mcount,case_count,model_counter,i1)=0;
end;
% Core CPI, data
if g_cpi_core_metrics_m_yoy(ff,i1)-g_cpi_core_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_cpi_core_data_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_cpi_core_data_direction(mcount,case_count,model_counter,i1)=0;
end;
%
if pce_switch;
% PCE, nowcast
if g_pcepm_nowcasts(mcount,case_count,model_counter,i1)-g_pcepm_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_pcepm_forecast_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_pcepm_forecast_direction(mcount,case_count,model_counter,i1)=0;
end;
% PCE, data
if g_pcepm_metrics_m_yoy(ff,i1)-g_pcepm_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_pcepm_data_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_pcepm_data_direction(mcount,case_count,model_counter,i1)=0;
end;
% Core PCE, nowcast
if g_pcepm_core_nowcasts(mcount,case_count,model_counter,i1)-g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_pcepm_core_forecast_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_pcepm_core_forecast_direction(mcount,case_count,model_counter,i1)=0;
end;
% Core PCE, data
if g_pcepm_core_metrics_m_yoy(ff,i1)-g_pcepm_core_nowcasts_last(mcount,case_count,model_counter,i1)>0; dfa_g_pcepm_core_data_direction(mcount,case_count,model_counter,i1)=1;
else; dfa_g_pcepm_core_data_direction(mcount,case_count,model_counter,i1)=0;
end;
%
end;
%
end;
end;


%---------------------------------------------------------------------
end; % case_count loop
%---------------------------------------------------------------------

%---------------------------------------------------------------------
end; % mcount loop
%---------------------------------------------------------------------

disp('-------------------------------------------------------------------------------');
disp('Case 1:');
disp('right before beginning of month,');
disp('no gasoline or oil price data,');
disp('missing 2 months of CPI data,');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 2:');
disp('day 8 of the month,');
disp('some gasoline and oil price data,');
disp('missing 2 months of CPI data,');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 3:');
disp('day 15 of the month,');
disp('some gasoline and oil price data');
disp('assume get CPI for previous month on 15th (missing 1 month),');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 4:');
disp('day 22 of the month,');
disp('some gasoline and oil price data');
disp('missing 1 month of CPI data,');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 5:');
disp('last day of the month,');
disp('all gasoline and oil price data,');
disp('missing 1 month of CPI data,');
disp('assume get PCE for previous month on last day (missing 1 month).');
disp('-------------------------------------------------------------------------------');
disp('Case 6:');
disp('middle of the following month,');
disp('all gasoline and oil price data,');
disp('assume get CPI for previous month on 15th (missing 0 months),');
disp('missing 1 month of PCE data');
disp('-------------------------------------------------------------------------------');

%---------------------------------------------------------------------
% Directional forecast accuracy
%---------------------------------------------------------------------
pt_tests_m_cpi=NaN(3,no_cases_m,1+no_alt_cases,2);
pt_tests_m_cpi_core=NaN(3,no_cases_m,1+no_alt_cases,2);
pt_tests_m_pcepm=NaN(3,no_cases_m,1+no_alt_cases,2);
pt_tests_m_pcepm_core=NaN(3,no_cases_m,1+no_alt_cases,2);
for i3=1:2;
for i2=1:1+no_alt_cases;
for i1=1:no_cases_m;
	f1=dfa_g_cpi_forecast_direction(:,i1,i2,i3)~=junk_nos & dfa_g_cpi_data_direction(:,i1,i2,i3)~=junk_nos;
	[pt_tests_m_cpi(1,i1,i2,i3),pt_tests_m_cpi(2,i1,i2,i3),pt_tests_m_cpi(3,i1,i2,i3)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_forecast_direction(f1,i1,i2,i3),dfa_g_cpi_data_direction(f1,i1,i2,i3));
	f1=dfa_g_cpi_core_forecast_direction(:,i1,i2,i3)~=junk_nos & dfa_g_cpi_core_data_direction(:,i1,i2,i3)~=junk_nos;
	[pt_tests_m_cpi_core(1,i1,i2,i3),pt_tests_m_cpi_core(2,i1,i2,i3),pt_tests_m_cpi_core(3,i1,i2,i3)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_core_forecast_direction(f1,i1,i2,i3),dfa_g_cpi_core_data_direction(f1,i1,i2,i3));
	f1=dfa_g_pcepm_forecast_direction(:,i1,i2,i3)~=junk_nos & dfa_g_pcepm_data_direction(:,i1,i2,i3)~=junk_nos;
	[pt_tests_m_pcepm(1,i1,i2,i3),pt_tests_m_pcepm(2,i1,i2,i3),pt_tests_m_pcepm(3,i1,i2,i3)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcepm_forecast_direction(f1,i1,i2,i3),dfa_g_pcepm_data_direction(f1,i1,i2,i3));
	f1=dfa_g_pcepm_core_forecast_direction(:,i1,i2,i3)~=junk_nos & dfa_g_pcepm_core_data_direction(:,i1,i2,i3)~=junk_nos;
	[pt_tests_m_pcepm_core(1,i1,i2,i3),pt_tests_m_pcepm_core(2,i1,i2,i3),pt_tests_m_pcepm_core(3,i1,i2,i3)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcepm_core_forecast_direction(f1,i1,i2,i3),dfa_g_pcepm_core_data_direction(f1,i1,i2,i3));
end;
end;
end;


%---------------------------------------------------------------------
% Compute forecast errors
%---------------------------------------------------------------------
rmses_m_cpi=NaN*ones(1+no_alt_cases,no_cases_m,2);
rmses_m_cpi_core=NaN*ones(1+no_alt_cases,no_cases_m,2);
gwstat_sig_m_cpi_comp=NaN(1+no_alt_cases,no_cases_m,2);
gwstat_sig_m_cpi_core_comp=NaN(1+no_alt_cases,no_cases_m,2);
if pce_switch;
rmses_m_pcepm=NaN*ones(1+no_alt_cases,no_cases_m,2);
rmses_m_pcepm_core=NaN*ones(1+no_alt_cases,no_cases_m,2);
gwstat_sig_m_pcepm_comp=NaN(1+no_alt_cases,no_cases_m,2);
gwstat_sig_m_pcepm_core_comp=NaN(1+no_alt_cases,no_cases_m,2);
end;
for i3=1:2;
for i2=1:1+no_alt_cases;
for i1=1:no_cases_m;
	% CPI
	f2=g_cpi_nowcasts(:,i1,i2,i3);
	a1=g_cpi_metrics_m_yoy(:,i3);
	f3=f2~=junk_nos & a1~=junk_nos;
	r1=sqrt(mean((f2(f3)-a1(f3)).^2));
	rmses_m_cpi(i2,i1,i3)=r1;
	if i2>1;
		f1=g_cpi_nowcasts(:,i1,1,i3);
		f3=f1~=junk_nos & f2~=junk_nos & a1~=junk_nos;
		% errors
		e1=f1(f3)-a1(f3);
		e2=f2(f3)-a1(f3);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_m_cpi_comp(i2,i1,i3)]=CPAtest(e1.^2-e2.^2,ones(sum(f3),1),1);
	end;
	%
	% Core CPI
	f2=g_cpi_core_nowcasts(:,i1,i2,i3);
	a1=g_cpi_core_metrics_m_yoy(:,i3);
	f3=f2~=junk_nos & a1~=junk_nos;
	r1=sqrt(mean((f2(f3)-a1(f3)).^2));
	rmses_m_cpi_core(i2,i1,i3)=r1;
	if i2>1;
		f1=g_cpi_core_nowcasts(:,i1,1,i3);
		f3=f1~=junk_nos & f2~=junk_nos & a1~=junk_nos;
		% errors
		e1=f1(f3)-a1(f3);
		e2=f2(f3)-a1(f3);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_m_cpi_core_comp(i2,i1,i3)]=CPAtest(e1.^2-e2.^2,ones(sum(f3),1),1);
	end;
	%
	if pce_switch;
	%
	% PCE
	f2=g_pcepm_nowcasts(:,i1,i2,i3);
	a1=g_pcepm_metrics_m_yoy(:,i3);
	f3=f2~=junk_nos & a1~=junk_nos & cond4 & cond6;
	r1=sqrt(mean((f2(f3)-a1(f3)).^2));
	rmses_m_pcepm(i2,i1,i3)=r1;
	if i2>1;
		f1=g_pcepm_nowcasts(:,i1,1,i3);
		f3=f1~=junk_nos & f2~=junk_nos & a1~=junk_nos & cond4 & cond6;
		% errors
		e1=f1(f3)-a1(f3);
		e2=f2(f3)-a1(f3);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_m_pcepm_comp(i2,i1,i3)]=CPAtest(e1.^2-e2.^2,ones(sum(f3),1),1);
	end;
	%
	% Core PCE
	f2=g_pcepm_core_nowcasts(:,i1,i2,i3);
	a1=g_pcepm_core_metrics_m_yoy(:,i3);
	f3=f2~=junk_nos & a1~=junk_nos & cond2 & cond6;
	r1=sqrt(mean((f2(f3)-a1(f3)).^2));
	rmses_m_pcepm_core(i2,i1,i3)=r1;
	if i2>1;
		f1=g_pcepm_core_nowcasts(:,i1,1,i3);
		f3=f1~=junk_nos & f2~=junk_nos & a1~=junk_nos & cond2 & cond6;
		% errors
		e1=f1(f3)-a1(f3);
		e2=f2(f3)-a1(f3);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_m_pcepm_core_comp(i2,i1,i3)]=CPAtest(e1.^2-e2.^2,ones(sum(f3),1),1);
	end;	
	%
	end; % pce_switch
	%
end;
end;
end;
% Table 3 data:
table_3=[rmses_m_cpi(:,:,1);rmses_m_cpi_core(:,:,1);rmses_m_pcepm(:,:,1);rmses_m_pcepm_core(:,:,1)];
table_3(1:8,6)=NaN; % CPI is known in case 6
% Table 3, statistical significance:
table_3_stat_sig=[gwstat_sig_m_cpi_comp(:,:,1); gwstat_sig_m_cpi_core_comp(:,:,1); gwstat_sig_m_pcepm_comp(:,:,1); gwstat_sig_m_pcepm_core_comp(:,:,1)];
% Figure 2 data:
figure_2=[table_3(1,:);table_3(5,:);table_3(9,:);table_3(13,:)];
% Table 4 data:
table_4=[squeeze(pt_tests_m_cpi(3,:,[1,3,4],1))';squeeze(pt_tests_m_cpi_core(3,:,[1,3,4],1))';squeeze(pt_tests_m_pcepm(3,:,[1,3,4],1))';squeeze(pt_tests_m_pcepm_core(3,:,[1,3,4],1))';...
	squeeze(pt_tests_m_cpi(2,:,[1,3,4],1))';squeeze(pt_tests_m_cpi_core(2,:,[1,3,4],1))';squeeze(pt_tests_m_pcepm(2,:,[1,3,4],1))';squeeze(pt_tests_m_pcepm_core(2,:,[1,3,4],1))'];
table_4(1:6,6)=NaN; % CPI is known in case 6
table_4(13:18,6)=NaN;
% Table 5 data:
table_5=[rmses_m_cpi(:,:,2);rmses_m_cpi_core(:,:,2);rmses_m_pcepm(:,:,2);rmses_m_pcepm_core(:,:,2)];
table_5(1:8,6)=NaN; % CPI is known in case 6
% Table 5, statistical significance:
table_5_stat_sig=[gwstat_sig_m_cpi_comp(:,:,2);gwstat_sig_m_cpi_core_comp(:,:,2);gwstat_sig_m_pcepm_comp(:,:,2);gwstat_sig_m_pcepm_core_comp(:,:,2)];