% nowcasting_inflation_quarterly.m
% Program to produce quarterly results.

% Load the data
nowcasting_inflation_setup;

%% Put in preliminaries here for MIDAS and dynamic factor model		-- here and elsewhere: comment out components from MIDAS and DFM to not reshare code but let the code run without incident 
%RootDir = cd;
%FunDir  = [RootDir,'/funcs/'];
%addpath(genpath(FunDir));

%---------------------------------------------------------------------
disp('-------------------------------------------------------------------------------');
disp('Quarterly nowcasting exercise');
disp('-------------------------------------------------------------------------------');
%---------------------------------------------------------------------
% For the quarterly nowcasting exercise, there are 15 key dates:
% 1. Day before the quarter begins--have no data for the current quarter, 
%    but assume you have CPI and PCE through the middle month of the 
%    previous quarter.
% 2. Day #8 of first month--have one week of data on oil and gasoline 
%    prices.
% 3. Day #15 of first month--assume that CPI for the third month of the 
% 	 previous quarter is released no later than this day, have two weeks
% 	 of gasoline and oil prices.
% 4. Day #22 of first month--have three weeks of oil and gasoline 
%    prices and CPI for third month of previous quarter
% 5. Final day of first month--assume that PCE for the third month of the
% 	 previous quarter is released no later than this day, have a full month
% 	 of oil and gasoline prices.
% 6. Day #8 of second month--have one month plus one week of data on oil
%    and gasoline prices, CPI and PCE data through end of previous 
%    quarter.
% 7. Day #15 of second month--assume that CPI for the first month of the
% 	 quarter of interest is released no later than this day, have one month 
% 	 plus two weeks of oil and gasoline prices.
% 8. Day #22 of second month--have one month plus three weeks of data on
%    oil and gasoline prices, plus CPI for first month of quarter.
% 9. Final day of second month--assume that PCE for the first month of the
% 	 quarter of interest is released no later than this day, have two full
% 	 months of oil and gasoline prices.
% 10. Day #8 of the third month--have two months plus one week of oil and
%     gasoline prices, CPI and PCE data through first month of current
%     quarter.
% 11. Day #15 of the third month--assume that CPI for the second month of 
% 	  the quarter of interest is released no later than this day, have two and
% 	  a half months of oil and gasoline data.
% 12. Day #22 of the third month--have two months plus three weeks of oil
%     and gasoline data, 2 months of CPI and 1 month of PCE.
% 13. Final day of the third month--assume that PCE for the second month
% 	  of the quarter of interest is released no later than this day, have all
% 	  three months of oil and gasoline data.
% 14. Middle of the first month in the following quarter--assume that CPI for 
% 	  the third month of the quarter of interest is released.
% 15. End of the first month in the following quarter--assume that PCE for 
% 	  the third month of the quarter of interest is released (note:
%     this won't be part of the nowcast set)
no_cases_q=14;

disp(['Quarter to start the analysis: ',num2str(start_quarter)]);
disp(['Quarter to end the analysis: ',num2str(end_quarter)]);

% set matrices to store inflation nowcasts
g_cpi_q_nowcasts=junk_nos_q*ones(1,no_cases_q);
g_cpi_core_q_nowcasts=junk_nos_q*ones(1,no_cases_q);
g_pcep_q_nowcasts=junk_nos_q*ones(1,no_cases_q);
g_pcep_core_q_nowcasts=junk_nos_q*ones(1,no_cases_q);

% Number of robustness exercises and matrices for inflation nowcasts
no_robustness_exercises=18;
g_cpi_q_nowcasts_robust=junk_nos*ones(length(junk_nos_q),no_cases_q,no_robustness_exercises);
g_cpi_core_q_nowcasts_robust=junk_nos*ones(length(junk_nos_q),no_cases_q,no_robustness_exercises);
g_pcep_q_nowcasts_robust=junk_nos*ones(length(junk_nos_q),no_cases_q,no_robustness_exercises);
g_pcep_core_q_nowcasts_robust=junk_nos*ones(length(junk_nos_q),no_cases_q,no_robustness_exercises);

% Quarterly statistical models
% Alternative cases to compare against the model
% 1. Inflation over the last four available quarters
% 2. AR(1) in gaps forecast (whole sample of Blue Chip data)
% 3. UCSV (Stock and Watson 2007) forecast (starting 1960)
no_alt_cases1=3;

% Other monthly nowcasting models
% 1. Small MIDAS model
% 2. Dynamic factor model (Modugno)
no_alt_cases2=2;

no_alt_cases=no_alt_cases1+no_alt_cases2;
%g_cpi_q_nowcasts_alt=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_cpi_core_q_nowcasts_alt=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_pcep_q_nowcasts_alt=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_pcep_core_q_nowcasts_alt=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_cpi_q_nowcasts_alt_last=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_cpi_core_q_nowcasts_alt_last=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_pcep_q_nowcasts_alt_last=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
%g_pcep_core_q_nowcasts_alt_last=junk_nos*ones(length(junk_nos_q),no_cases_q,no_alt_cases);
% Load results from alternative models
load quarterly_nowcasts;

% Directional forecast accuracy matrices
dfa_g_cpi_forecast_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_cpi_data_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_cpi_core_forecast_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_cpi_core_data_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_pcep_forecast_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_pcep_data_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_pcep_core_forecast_direction=junk_nos_q*ones(1,no_cases_q);
dfa_g_pcep_core_data_direction=junk_nos_q*ones(1,no_cases_q);
%
dfa_g_cpi_forecast_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_cpi_data_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_cpi_core_forecast_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_cpi_core_data_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_pcep_forecast_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_pcep_data_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_pcep_core_forecast_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);
dfa_g_pcep_core_data_direction_alt=junk_nos*ones(length(date_q),no_cases_q,no_alt_cases);

% Load Blue Chip long-run inflation estimates
bcei_lr_pce=[];
bcei_lr_cpi=[];
%[bcei_lr_raw,bcei_lr_raw_txt]=xlsread('nowcast_inflation_data','bcei_lr');
%%linearly_interpolate=1; % to linearly interpolate between observations
%linearly_interpolate=0; % to use last available observation
%[bcei_lr_pce,bcei_lr_cpi]=fn_clean_bcei_lr(bcei_lr_raw,date_q,linearly_interpolate);

%---------------------------------------------------------------------
for qcount=find(date_q==start_quarter):find(date_q==end_quarter);
%---------------------------------------------------------------------

disp(['Quarter = ',num2str(date_q(qcount))]);
yyyyq=date_q(qcount);
yy=round(yyyyq); % year of the quarter under investigation
mm=3*(round(10*rem(yyyyq,1))-1)+1; % first month of the quarter under investigation
yyyymm=yy+mm/100; % date of first month of the quarter under investigation
ff=find(date_m==yyyymm); % ff is the position of the first month of the quarter under investigation

%---------------------------------------------------------------------
for case_count=1:no_cases_q;
%---------------------------------------------------------------------

%---------------------------------------------------------------------
% Case 1: 
% right before beginning of quarter, 
% no gasoline or oil price data, 
% missing last month of previous quarter CPI data (missing 4 months),
% missing last month of previous quarter PCE data (missing 4 months)
%---------------------------------------------------------------------
if case_count==1;
date_cutoff=yy*10000+(mm-1)*100+31;
ff_cpi=ff-2; % have through middle month of previous quarter
ff_pcepm=ff-2; % have through middle month of previous quarter
end;

%---------------------------------------------------------------------
% Case 2: 
% day 8 of the first month, 
% some gasoline and oil price data, 
% missing last month of previous quarter CPI data (missing 4 months),
% missing last month of previous quarter PCE data (missing 4 months)
%---------------------------------------------------------------------
if case_count==2;
date_cutoff=yy*10000+(mm)*100+08;
ff_cpi=ff-2; % have through middle month of previous quarter
ff_pcepm=ff-2; % have through middle month of previous quarter
end;

%---------------------------------------------------------------------
% Case 3: 
% day 15 of the first month, 
% some gasoline and oil price data, 
% assume get CPI for last month of previous quarter (missing 3 months),
% missing last month of previous quarter PCE data (missing 4 months)
%---------------------------------------------------------------------
if case_count==3;
date_cutoff=yy*10000+(mm)*100+15;
ff_cpi=ff-1; % have through final month of previous quarter
ff_pcepm=ff-2; % have through middle month of previous quarter
end;

%---------------------------------------------------------------------
% Case 4: 
% day 22 of the first month, 
% some gasoline and oil price data, 
% missing first month of current quarter CPI data (missing 3 months),
% missing last month of previous quarter PCE data (missing 4 months)
%---------------------------------------------------------------------
if case_count==4;
date_cutoff=yy*10000+(mm)*100+22;
ff_cpi=ff-1; % have through final month of previous quarter
ff_pcepm=ff-2; % have through middle month of previous quarter
end;

%---------------------------------------------------------------------
% Case 5: 
% last day of the first month, 
% 1 month gasoline and oil price data, 
% missing first month of current quarter CPI data (missing 3 months),
% assume get PCE for last month of previous quarter (missing 3 month)
%---------------------------------------------------------------------
if case_count==5;
date_cutoff=yy*10000+(mm+1)*100+00;
ff_cpi=ff-1; % have through final month of previous quarter
ff_pcepm=ff-1; % have through final month of previous quarter
end;

%---------------------------------------------------------------------
% Case 6: 
% day #8 of the second month
% 1+ months gasoline and oil price data,
% missing first month of current quarter CPI data (missing 3 months),
% missing first month of current quarter PCE data (missing 3 months)
%---------------------------------------------------------------------
if case_count==6;
date_cutoff=yy*10000+(mm+1)*100+08;
ff_cpi=ff-1; % have through final month of previous quarter
ff_pcepm=ff-1; % have through final month of previous quarter
end;

%---------------------------------------------------------------------
% Case 7: 
% day #15 of the second month
% 1+ months gasoline and oil price data,
% assume get CPI for first month of current quarter (missing 2 months),
% missing first month of current quarter PCE data (missing 3 months)
%---------------------------------------------------------------------
if case_count==7;
date_cutoff=yy*10000+(mm+1)*100+15;
ff_cpi=ff; % have through first month of current quarter
ff_pcepm=ff-1; % have through final month of previous quarter
end;

%---------------------------------------------------------------------
% Case 8: 
% day #22 of the second month
% 1+ months gasoline and oil price data,
% missing second month of current quarter CPI data (missing 2 months),
% missing first month of current quarter PCE data (missing 3 months)
%---------------------------------------------------------------------
if case_count==8;
date_cutoff=yy*10000+(mm+1)*100+22;
ff_cpi=ff; % have through first month of current quarter
ff_pcepm=ff-1; % have through final month of previous quarter
end;

%---------------------------------------------------------------------
% Case 9: 
% last day of the second month
% 2 months gasoline and oil price data,
% missing second month of current quarter CPI data (missing 2 months),
% assume get PCE for first month of current quarter (missing 2 months)
%---------------------------------------------------------------------
if case_count==9;
date_cutoff=yy*10000+(mm+2)*100+00;
ff_cpi=ff; % have through first month of current quarter
ff_pcepm=ff; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Case 10: 
% day #8 of the third month
% 2+ months gasoline and oil price data,
% missing second month of current quarter CPI data (missing 2 months),
% missing second month of current quarter PCE data (missing 2 months)
%---------------------------------------------------------------------
if case_count==10;
date_cutoff=yy*10000+(mm+2)*100+08;
ff_cpi=ff; % have through first month of current quarter
ff_pcepm=ff; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Case 11: 
% day #15 of the third month
% 2+ months gasoline and oil price data,
% assume get CPI for second month of current quarter (missing 1 month),
% missing second month of current quarter PCE data (missing 2 months)
%---------------------------------------------------------------------
if case_count==11;
date_cutoff=yy*10000+(mm+2)*100+15;
ff_cpi=ff+1; % have through second month of current quarter
ff_pcepm=ff; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Case 12:
% day #22 of the third month
% 2+ months gasoline and oil price data,
% missing third month of current quarter CPI data (missing 1 month),
% missing second month of current quarter PCE data (missing 2 months)
%---------------------------------------------------------------------
if case_count==12;
date_cutoff=yy*10000+(mm+2)*100+22;
ff_cpi=ff+1; % have through second month of current quarter
ff_pcepm=ff; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Case 13:
% last day of the third month
% 3 months gasoline and oil price data,
% missing third month of current quarter CPI data (missing 1 month),
% assume get PCE data for second month of current quarter (missing 1 month)
%---------------------------------------------------------------------
if case_count==13;
date_cutoff=yy*10000+(mm+3)*100+00;
ff_cpi=ff+1; % have through second month of current quarter
ff_pcepm=ff+1; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Case 14:
% middle of first month of the following quarter
% 3 months gasoline and oil price data,
% assume get CPI data for third month of current quarter (missing 0 months)
% missing third month of current quarter PCE data (missing 1 month)
%---------------------------------------------------------------------
if case_count==14;
date_cutoff=yy*10000+(mm+3)*100+15;
ff_cpi=ff+2; % have through second month of current quarter
ff_pcepm=ff+1; % have through first month of previous quarter
end;

%---------------------------------------------------------------------
% Set up available data
%---------------------------------------------------------------------
wcount=max(find(date_w<=date_cutoff));
dcount=max(find(date_d<=date_cutoff));
f1=min(find(cpi_rt(ff_cpi+1,2:end)~=junk_nos));
mcount_cpi=1+f1;
if pce_switch;
f1=min(find(pcepm_rt(ff_pcepm+1,2:end)~=junk_nos));
mcount_pcepm=1+f1;
end;
%
p_oil_d=junk_nos*ones(size(date_d));
p_oil_d(1:dcount)=p_oil_d_rt(1:dcount,2);
p_oil_wti_d=junk_nos*ones(size(date_d));
p_oil_wti_d(1:dcount)=p_oil_wti_d_rt(1:dcount,2);
p_foodstuffs_d=junk_nos*ones(size(date_d));
p_foodstuffs_d(1:dcount)=p_foodstuffs_d_rt(1:dcount,2);
%p_all_commodities_d=junk_nos*ones(size(date_d,1),size(p_all_commodities_d_rt,2)-1);
%p_all_commodities_d(1:dcount,:)=p_all_commodities_d_rt(1:dcount,2:end);
%
p_gasoline_w=junk_nos*ones(size(date_w));
p_gasoline_w(1:wcount)=p_gasoline_w_rt(1:wcount,2);
%p_gasoline_by_grade_w=junk_nos*ones(size(date_w,1),size(p_gasoline_by_grade_w_rt,2)-1);
%p_gasoline_by_grade_w(1:wcount,:)=p_gasoline_by_grade_w_rt(1:wcount,2:end);
%
cpi=cpi_rt(2:end,mcount_cpi);
[g_cpi,g_cpi_dln]=fn_compute_g_monthly(cpi);
cpi_core=cpi_core_rt(2:end,mcount_cpi);
[g_cpi_core,g_cpi_core_dln]=fn_compute_g_monthly(cpi_core);
cpi_food=cpi_food_rt(2:end,mcount_cpi);
[g_cpi_food,g_cpi_food_dln]=fn_compute_g_monthly(cpi_food);
cpi_gasoline=cpi_gasoline_rt(2:end,mcount_cpi);
[g_cpi_gasoline,g_cpi_gasoline_dln]=fn_compute_g_monthly(cpi_gasoline);
% unfortunately, don't have real-time CPI--food at home (which 
% conceptually matches up with PCE--goods--food), use CPI--food instead
g_cpi_food_at_home=g_cpi_food;
%
if pce_switch;
pcepm=pcepm_rt(2:end,mcount_pcepm);
[g_pcepm,g_pcepm_dln]=fn_compute_g_monthly(pcepm);
pcepm_core=pcepm_core_rt(2:end,mcount_pcepm);
[g_pcepm_core,g_pcepm_core_dln]=fn_compute_g_monthly(pcepm_core);
% unfortunately, don't have real-time PCE--goods--food, use CPI--food instead
g_pcepm_goods_food=g_cpi_food; 
end; % pce_switch

%---------------------------------------------------------------------
% Make nowcasts
%---------------------------------------------------------------------
f_cpi=cpi;
f_cpi_core=cpi_core;
f_pcepm=pcepm;
f_pcepm_core=pcepm_core;
[f_g_cpi,f_g_cpi_core,f_g_pcepm,f_g_pcepm_core]=fn_make_nowcasts(date_m,date_d,date_w,...
	p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	

%-------------------------------
% Convert growth rates to levels
%-------------------------------
f2=max(find(cpi~=junk_nos));
for i1=f2+1:ff+2;
	f_cpi(i1)=f_cpi(i1-1)*(1+f_g_cpi(i1)/100);
	f_cpi_core(i1)=f_cpi_core(i1-1)*(1+f_g_cpi_core(i1)/100);
end;
if pce_switch;
f1=max(find(pcepm~=junk_nos));
for i1=f1+1:ff+2;
	f_pcepm(i1)=f_pcepm(i1-1)*(1+f_g_pcepm(i1)/100);
	f_pcepm_core(i1)=f_pcepm_core(i1-1)*(1+f_g_pcepm_core(i1)/100);
end;
end; % pce_switch

%------------------
% Save the nowcasts
%------------------
g_cpi_q_nowcasts(qcount,case_count)=100*((mean(f_cpi(ff:ff+2))/mean(f_cpi(ff-3:ff-1)))^4-1);
g_cpi_core_q_nowcasts(qcount,case_count)=100*((mean(f_cpi_core(ff:ff+2))/mean(f_cpi_core(ff-3:ff-1)))^4-1);
if pce_switch;
g_pcep_q_nowcasts(qcount,case_count)=100*((mean(f_pcepm(ff:ff+2))/mean(f_pcepm(ff-3:ff-1)))^4-1);
g_pcep_core_q_nowcasts(qcount,case_count)=100*((mean(f_pcepm_core(ff:ff+2))/mean(f_pcepm_core(ff-3:ff-1)))^4-1);
end; % pce_switch

%------------------------------
% Directional forecast accuracy
%------------------------------
% CPI, nowcast
if 100*((mean(f_cpi(ff:ff+2))/mean(f_cpi(ff-3:ff-1)))^4-1)-100*((mean(f_cpi(ff-3:ff-1))/mean(f_cpi(ff-6:ff-4)))^4-1)>0; dfa_g_cpi_forecast_direction(qcount,case_count)=1;
else; dfa_g_cpi_forecast_direction(qcount,case_count)=0;
end;
% CPI, data
if g_cpi_q_metric(qcount)-100*((mean(f_cpi(ff-3:ff-1))/mean(f_cpi(ff-6:ff-4)))^4-1)>0; dfa_g_cpi_data_direction(qcount,case_count)=1;
else; dfa_g_cpi_data_direction(qcount,case_count)=0;
end;
% Core CPI, nowcast
if 100*((mean(f_cpi_core(ff:ff+2))/mean(f_cpi_core(ff-3:ff-1)))^4-1)-100*((mean(f_cpi_core(ff-3:ff-1))/mean(f_cpi_core(ff-6:ff-4)))^4-1)>0; dfa_g_cpi_core_forecast_direction(qcount,case_count)=1;
else; dfa_g_cpi_core_forecast_direction(qcount,case_count)=0;
end;
% Core CPI, data
if g_cpi_core_q_metric(qcount)-100*((mean(f_cpi_core(ff-3:ff-1))/mean(f_cpi_core(ff-6:ff-4)))^4-1)>0; dfa_g_cpi_core_data_direction(qcount,case_count)=1;
else; dfa_g_cpi_core_data_direction(qcount,case_count)=0;
end;
%
if pce_switch;
% PCE, nowcast
if 100*((mean(f_pcepm(ff:ff+2))/mean(f_pcepm(ff-3:ff-1)))^4-1)-100*((mean(f_pcepm(ff-3:ff-1))/mean(f_pcepm(ff-6:ff-4)))^4-1)>0; dfa_g_pcep_forecast_direction(qcount,case_count)=1;
else; dfa_g_pcep_forecast_direction(qcount,case_count)=0;
end;
% PCE, data
if g_pcep_q_metric(qcount)-100*((mean(f_pcepm(ff-3:ff-1))/mean(f_pcepm(ff-6:ff-4)))^4-1)>0; dfa_g_pcep_data_direction(qcount,case_count)=1;
else; dfa_g_pcep_data_direction(qcount,case_count)=0;
end;
% Core PCE, nowcast
if 100*((mean(f_pcepm_core(ff:ff+2))/mean(f_pcepm_core(ff-3:ff-1)))^4-1)-100*((mean(f_pcepm_core(ff-3:ff-1))/mean(f_pcepm_core(ff-6:ff-4)))^4-1)>0; dfa_g_pcep_core_forecast_direction(qcount,case_count)=1;
else; dfa_g_pcep_core_forecast_direction(qcount,case_count)=0;
end;
% Core PCE, data
if g_pcep_core_q_metric(qcount)-100*((mean(f_pcepm_core(ff-3:ff-1))/mean(f_pcepm_core(ff-6:ff-4)))^4-1)>0; dfa_g_pcep_core_data_direction(qcount,case_count)=1;
else; dfa_g_pcep_core_data_direction(qcount,case_count)=0;
end;
%
end;
	
%---------------------------------------------------------------------
% Robustness exercises
%---------------------------------------------------------------------
% Set up data
f_cpi_robust=cpi*ones(1,no_robustness_exercises);
f_cpi_core_robust=cpi_core*ones(1,no_robustness_exercises);
f_pcepm_robust=pcepm*ones(1,no_robustness_exercises);
f_pcepm_core_robust=pcepm_core*ones(1,no_robustness_exercises);
f_g_cpi_robust=junk_nos_m*ones(1,no_robustness_exercises);
f_g_cpi_core_robust=junk_nos_m*ones(1,no_robustness_exercises);
f_g_pcepm_robust=junk_nos_m*ones(1,no_robustness_exercises);
f_g_pcepm_core_robust=junk_nos_m*ones(1,no_robustness_exercises);
%
%%%%%%%%%%%%%
% Exercise 1:
% Set tau=12 months
robust_counter=1;
tau_old=tau; % save old tau
tau=12;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau=tau_old; % reset tau
%
%%%%%%%%%%%%%
% Exercise 2:
% Set tau=36 months
robust_counter=2;
tau_old=tau; % save old tau
tau=36;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau=tau_old; % reset tau
%
%%%%%%%%%%%%%
% Exercise 3:
% Set tau=120 months
robust_counter=3;
tau_old=tau; % save old tau
tau=120;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau=tau_old; % reset tau
%
%%%%%%%%%%%%%
% Exercise 4:
% Set tau_L=48 months
robust_counter=4;
tau_L_old=tau_L; % save old tau_L
tau_L=48;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau_L=tau_L_old; % reset tau_L
%
%%%%%%%%%%%%%
% Exercise 5:
% Set tau_L=72 months
robust_counter=5;
tau_L_old=tau_L; % save old tau_L
tau_L=72;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau_L=tau_L_old; % reset tau_L
%
%%%%%%%%%%%%%
% Exercise 6:
% Set tau_L=entire expanding real-time sample
robust_counter=6;
tau_L_old=tau_L; % save old tau_L
tau_L=1000;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
tau_L=tau_L_old; % reset tau_L
%
%%%%%%%%%%%%%
% Exercise 7:
% Set J=6 months
robust_counter=7;
J_old=J_; % save old J
J_=6;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
J_=J_old; % reset J
%
%%%%%%%%%%%%%
% Exercise 8:
% Set J=24 months
robust_counter=8;
J_old=J_; % save old J
J_=24;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
J_=J_old; % reset J
%
%%%%%%%%%%%%%
% Exercise 9:
% Set J=36 months
robust_counter=9;
J_old=J_; % save old J_
J_=36;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
J_=J_old; % reset J_
%
%%%%%%%%%%%%%
% Exercise 10:
% Bridge CRB food prices to nowcast food inflation (using CRB foodstuffs inflation SA at time t-k and food inflation at t-1 to forecast food inflation at t, where k is chosen to maximize R^2)
robust_counter=10;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_crb_foodstuffs(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,p_foodstuffs_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 11:
% Drop food inflation
robust_counter=11;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_food_inflation(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 12:
% Use WTI crude oil prices instead of Brent crude oil prices
robust_counter=12;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 13:
% Drop oil prices as a predictor of gasoline inflation
robust_counter=13;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_oil(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 14:
% Drop gasoline inflation
robust_counter=14;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_gasoline(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 14:
% Drop gasoline inflation
robust_counter=14;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_gasoline(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 15:
% Extend oil prices as a predictor of gasoline prices
robust_counter=15;
extra_months_old=extra_months; % save old extra_months
extra_months=3;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
extra_months=extra_months_old; % reset extra_months
%
%%%%%%%%%%%%%
% Exercise 16:
% Drop bridging equations from CPI to PCE
robust_counter=16;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_bridging(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
%
%%%%%%%%%%%%%
% Exercise 17:
% Single models, no model switching (extend oil prices and drop bridging)
robust_counter=17;
extra_months_old=extra_months; % save old extra_months
extra_months=3;
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_drop_bridging(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
extra_months=extra_months_old;
%
%%%%%%%%%%%%%
% Exercise 18:
% Use AR(1) instead of moving averages
robust_counter=18;
J_old=J_; % save old J_
J_=1; % J_ in this function defines AR(J_)
[f_g_cpi_robust(:,robust_counter),f_g_cpi_core_robust(:,robust_counter),f_g_pcepm_robust(:,robust_counter),f_g_pcepm_core_robust(:,robust_counter)]=...
		fn_make_nowcasts_with_ar(date_m,date_d,date_w,...
		p_gasoline_w,p_oil_wti_d,g_cpi,g_cpi_core,g_cpi_food,g_cpi_gasoline,g_cpi_food_at_home,g_pcepm,g_pcepm_core,g_pcepm_goods_food,pce_switch);	
J_=J_old; % reset J_
%
% Convert growth rates to levels
%
f2=max(find(cpi~=junk_nos));
for robust_counter=1:no_robustness_exercises;
%
for i1=f2+1:ff+2;
	f_cpi_robust(i1,robust_counter)=f_cpi_robust(i1-1,robust_counter)*(1+f_g_cpi_robust(i1,robust_counter)/100);
	f_cpi_core_robust(i1,robust_counter)=f_cpi_core_robust(i1-1,robust_counter)*(1+f_g_cpi_core_robust(i1,robust_counter)/100);
end;
if pce_switch;
f1=max(find(pcepm~=junk_nos));
for i1=f1+1:ff+2;
	f_pcepm_robust(i1,robust_counter)=f_pcepm_robust(i1-1,robust_counter)*(1+f_g_pcepm_robust(i1,robust_counter)/100);
	f_pcepm_core_robust(i1,robust_counter)=f_pcepm_core_robust(i1-1,robust_counter)*(1+f_g_pcepm_core_robust(i1,robust_counter)/100);
end;
end; % pce_switch
%
end;
%
% Save the robustness exercise nowcasts
%
for robust_counter=1:no_robustness_exercises;
%
g_cpi_q_nowcasts_robust(qcount,case_count,robust_counter)=100*((mean(f_cpi_robust(ff:ff+2,robust_counter))/mean(f_cpi_robust(ff-3:ff-1,robust_counter)))^4-1);
g_cpi_core_q_nowcasts_robust(qcount,case_count,robust_counter)=100*((mean(f_cpi_core_robust(ff:ff+2,robust_counter))/mean(f_cpi_core_robust(ff-3:ff-1,robust_counter)))^4-1);
if pce_switch;
g_pcep_q_nowcasts_robust(qcount,case_count,robust_counter)=100*((mean(f_pcepm_robust(ff:ff+2,robust_counter))/mean(f_pcepm_robust(ff-3:ff-1,robust_counter)))^4-1);
g_pcep_core_q_nowcasts_robust(qcount,case_count,robust_counter)=100*((mean(f_pcepm_core_robust(ff:ff+2,robust_counter))/mean(f_pcepm_core_robust(ff-3:ff-1,robust_counter)))^4-1);
end; % pce_switch
%
end;


%---------------------------------------------------------------------
% Make alternative forecasts
%---------------------------------------------------------------------
%if case_count==1; % run all alternative forecasts
%[g_cpi_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_cpi_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,cpi,bcei_lr_cpi);
%[g_cpi_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_cpi_core_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,cpi_core,bcei_lr_cpi);
%cpi_q_last=fn_mtoqavg(cpi,date_m,date_q);
%if pce_switch;
%[g_pcep_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_pcep_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,pcepm,bcei_lr_pce);
%[g_pcep_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_pcep_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,pcepm_core,bcei_lr_pce);
%pcep_q_last=fn_mtoqavg(pcepm,date_m,date_q);
%end; % pce_switch
%else; % run alternative forecasts selectively if something has changed
%cpi_q=fn_mtoqavg(cpi,date_m,date_q);
%if sum(cpi_q~=cpi_q_last)>0;
%	[g_cpi_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_cpi_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,cpi,bcei_lr_cpi);
%	[g_cpi_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_cpi_core_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,cpi_core,bcei_lr_cpi);
%	cpi_q_last=cpi_q;
%else;
%	g_cpi_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1)=g_cpi_q_nowcasts_alt(qcount,case_count-1,1:no_alt_cases1);
%	g_cpi_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)=g_cpi_q_nowcasts_alt_last(qcount,case_count-1,1:no_alt_cases1);
%	g_cpi_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1)=g_cpi_core_q_nowcasts_alt(qcount,case_count-1,1:no_alt_cases1);
%	g_cpi_core_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)=g_cpi_core_q_nowcasts_alt_last(qcount,case_count-1,1:no_alt_cases1);
%end;
%pcep_q=fn_mtoqavg(pcepm,date_m,date_q);
%if sum(pcep_q~=pcep_q_last)>0;
%	[g_pcep_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_pcep_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,pcepm,bcei_lr_pce);
%	[g_pcep_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1),g_pcep_core_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)]=fn_make_forecast_alternatives_q(date_q,qcount,date_m,pcepm_core,bcei_lr_pce);
%	pcep_q_last=pcep_q;
%else;
%	g_pcep_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1)=g_pcep_q_nowcasts_alt(qcount,case_count-1,1:no_alt_cases1);
%	g_pcep_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)=g_pcep_q_nowcasts_alt_last(qcount,case_count-1,1:no_alt_cases1);
%	g_pcep_core_q_nowcasts_alt(qcount,case_count,1:no_alt_cases1)=g_pcep_core_q_nowcasts_alt(qcount,case_count-1,1:no_alt_cases1);
%	g_pcep_core_q_nowcasts_alt_last(qcount,case_count,1:no_alt_cases1)=g_pcep_core_q_nowcasts_alt_last(qcount,case_count-1,1:no_alt_cases1);
%end;
%end;


%---------------------------------------------------------------------
% Make other nowcasts from competing monthly models
%---------------------------------------------------------------------


%%--------------------------------------
%% Other nowcasting model: MIDAS (small)
%%--------------------------------------
%other_counter=1;
%% make the nowcasts
%% using natural log monthly growth rates inside the model and as outputs; need to convert
%%
%% CPI
%daily_data_factor=[date_d,p_oil_d];
%daily_series={'crudeoil'}; 
%daily_transform=ones(1,1); %1=take growth rates; 0=already stationary; 2=differences
%weekly_data_factor=[date_w,p_gasoline_w];
%weekly_series={'reggrade'};
%weekly_transform=ones(1,1); %1=take growth rates; 0=already stationary 2=differences
%monthly_data_factor=[date_m,g_cpi_dln,g_cpi_core_dln,g_cpi_gasoline_dln,g_cpi_food_dln];
%monthly_series={'cpi','cpi_core','cpi_gasoline','cpi_food'};
%monthly_transform=zeros(4,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1; 
%no_months_to_do=(ff+2)-max(find(g_cpi~=junk_nos));
%[f_g_cpi_dln]=fn_make_nowcasts_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);
%%
%% Core CPI
%monthly_data_factor=[date_m,g_cpi_core_dln,g_cpi_dln];
%monthly_series={'cpi_core','cpi'};
%monthly_transform=zeros(2,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1;
%no_months_to_do=(ff+2)-max(find(g_cpi_core~=junk_nos));
%[f_g_cpi_core_dln]=fn_make_nowcasts_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);
%%
%% PCE
%monthly_data_factor=[date_m,g_pcepm_dln,g_cpi_dln,g_pcepm_core_dln,g_cpi_gasoline_dln,g_cpi_food_dln];
%monthly_series={'pce','cpi','core_pce','cpi_gasoline','cpi_food'};
%monthly_transform=zeros(5,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1;
%no_months_to_do=(ff+2)-max(find(g_pcepm~=junk_nos));
%[f_g_pcepm_dln]=fn_make_nowcasts_PCE_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);	
%%
%% Core PCE
%monthly_data_factor=[date_m,g_pcepm_core_dln,g_cpi_core_dln];
%monthly_series={'core_pce','core_cpi'};
%monthly_transform=zeros(2,1);
%midas_lags=1;
%use_Daily=1; 
%use_Weekly=1;
%no_months_to_do=(ff+2)-max(find(g_pcepm_core~=junk_nos));
%[f_g_pcepm_core_dln]=fn_make_nowcasts_PCE_MIDASmodel(daily_data_factor,weekly_data_factor,monthly_data_factor,midas_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform,use_Daily,use_Weekly,no_months_to_do);
%%
%% Convert growth rates to levels
%f_cpi=cpi;
%f_cpi_core=cpi_core;
%f_pcepm=pcepm;
%f_pcepm_core=pcepm_core;
%f2=max(find(cpi~=junk_nos));
%for i1=f2+1:ff+2;
%	f_cpi(i1)=exp(log(f_cpi(i1-1))+f_g_cpi_dln(i1)/100);
%	f_cpi_core(i1)=exp(log(f_cpi_core(i1-1))+f_g_cpi_core_dln(i1)/100);
%end;
%if pce_switch;
%f1=max(find(pcepm~=junk_nos));
%for i1=f1+1:ff+2;
%	f_pcepm(i1)=exp(log(f_pcepm(i1-1))+f_g_pcepm_dln(i1)/100);
%	f_pcepm_core(i1)=exp(log(f_pcepm_core(i1-1))+f_g_pcepm_core_dln(i1)/100);
%end;
%end; % pce_switch
%%
%% Save the nowcasts
%g_cpi_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi(ff:ff+2))/mean(f_cpi(ff-3:ff-1)))^4-1);
%g_cpi_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi(ff-3:ff-1))/mean(f_cpi(ff-6:ff-4)))^4-1);
%g_cpi_core_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi_core(ff:ff+2))/mean(f_cpi_core(ff-3:ff-1)))^4-1);
%g_cpi_core_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi_core(ff-3:ff-1))/mean(f_cpi_core(ff-6:ff-4)))^4-1);
%if pce_switch;
%g_pcep_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm(ff:ff+2))/mean(f_pcepm(ff-3:ff-1)))^4-1);
%g_pcep_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm(ff-3:ff-1))/mean(f_pcepm(ff-6:ff-4)))^4-1);
%g_pcep_core_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm_core(ff:ff+2))/mean(f_pcepm_core(ff-3:ff-1)))^4-1);
%g_pcep_core_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm_core(ff-3:ff-1))/mean(f_pcepm_core(ff-6:ff-4)))^4-1);
%end; % pce_switch


%%---------------------------------------------------------------------
%% Other nowcasting model: Dynamic factor model
%%---------------------------------------------------------------------
%other_counter=2;
%% make the nowcasts
%% using natural log monthly growth rates inside the model and as outputs; need to convert
%%
%% Large version, similar to Modugno (2013):
%daily_data_factor=[date_d,[p_oil_d,p_all_commodities_d(:,[2:14])]];
%daily_series={'crudeoil','foodstuffs','cereal','foodoils','sugar','rawindustrials','rawag','spinning','metals','steel','treas10yr','treas3mo','sp500','fxmajor'};
%daily_transform=ones(14,1); %1=take growth rates; 0=already stationary; 2=differences
%daily_transform(11,1)=2; daily_transform(12,1)=2; %only differences (not logged) since they are interest rates
%weekly_data_factor=[date_w,p_gasoline_by_grade_w];
%weekly_series={'reggrade','midgrade','premium','diesel'};
%weekly_transform=ones(4,1); %1=take growth rates; 0=already stationary
%monthly_data_factor=[date_m,g_cpi_dln,g_cpi_core_dln,g_pcepm_dln,g_pcepm_core_dln,g_cpi_food_dln,g_cpi_gasoline_dln];
%monthly_series={'cpi','corecpi','pce','corepce','cpi_food','cpi_gasoline'};
%monthly_transform=zeros(6,1);
%factor_lags=6;
%[f_g_cpi_dln,f_g_cpi_core_dln,f_g_pcepm_dln,f_g_pcepm_core_dln]=fn_make_nowcasts_factormodel(daily_data_factor,weekly_data_factor,monthly_data_factor,factor_lags,...
%	daily_series,daily_transform,weekly_series,weekly_transform,monthly_series,monthly_transform);
%%
%% Convert growth rates to levels
%f_cpi=cpi;
%f_cpi_core=cpi_core;
%f_pcepm=pcepm;
%f_pcepm_core=pcepm_core;
%f2=max(find(cpi~=junk_nos));
%for i1=f2+1:ff+2;
%	f_cpi(i1)=exp(log(f_cpi(i1-1))+f_g_cpi_dln(i1)/100);
%	f_cpi_core(i1)=exp(log(f_cpi_core(i1-1))+f_g_cpi_core_dln(i1)/100);
%end;
%if pce_switch;
%f1=max(find(pcepm~=junk_nos));
%for i1=f1+1:ff+2;
%	f_pcepm(i1)=exp(log(f_pcepm(i1-1))+f_g_pcepm_dln(i1)/100);
%	f_pcepm_core(i1)=exp(log(f_pcepm_core(i1-1))+f_g_pcepm_core_dln(i1)/100);
%end;
%end; % pce_switch
%%
%% Save the nowcasts
%g_cpi_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi(ff:ff+2))/mean(f_cpi(ff-3:ff-1)))^4-1);
%g_cpi_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi(ff-3:ff-1))/mean(f_cpi(ff-6:ff-4)))^4-1);
%g_cpi_core_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi_core(ff:ff+2))/mean(f_cpi_core(ff-3:ff-1)))^4-1);
%g_cpi_core_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_cpi_core(ff-3:ff-1))/mean(f_cpi_core(ff-6:ff-4)))^4-1);
%if pce_switch;
%g_pcep_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm(ff:ff+2))/mean(f_pcepm(ff-3:ff-1)))^4-1);
%g_pcep_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm(ff-3:ff-1))/mean(f_pcepm(ff-6:ff-4)))^4-1);
%g_pcep_core_q_nowcasts_alt(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm_core(ff:ff+2))/mean(f_pcepm_core(ff-3:ff-1)))^4-1);
%g_pcep_core_q_nowcasts_alt_last(qcount,case_count,no_alt_cases1+other_counter)=100*((mean(f_pcepm_core(ff-3:ff-1))/mean(f_pcepm_core(ff-6:ff-4)))^4-1);
%end; % pce_switch


%---------------------------------------------------------------------
% Directional forecast accuracy of alternative models
%---------------------------------------------------------------------
for i1=1:no_alt_cases;
	% CPI, nowcast
	if g_cpi_q_nowcasts_alt(qcount,case_count,i1)-g_cpi_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_cpi_forecast_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_cpi_forecast_direction_alt(qcount,case_count,i1)=0;
	end;
	% CPI, data
	if g_cpi_q_metric(qcount)-g_cpi_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_cpi_data_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_cpi_data_direction_alt(qcount,case_count,i1)=0;
	end;
	% Core CPI, nowcast
	if g_cpi_core_q_nowcasts_alt(qcount,case_count,i1)-g_cpi_core_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_cpi_core_forecast_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_cpi_core_forecast_direction_alt(qcount,case_count,i1)=0;
	end;
	% Core CPI, data
	if g_cpi_core_q_metric(qcount)-g_cpi_core_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_cpi_core_data_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_cpi_core_data_direction_alt(qcount,case_count,i1)=0;
	end;
	if pce_switch;
	% PCE, nowcast
	if g_pcep_q_nowcasts_alt(qcount,case_count,i1)-g_pcep_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_pcep_forecast_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_pcep_forecast_direction_alt(qcount,case_count,i1)=0;
	end;
	% PCE, data
	if g_pcep_q_metric(qcount)-g_pcep_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_pcep_data_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_pcep_data_direction_alt(qcount,case_count,i1)=0;
	end;
	% Core PCE, nowcast
	if g_pcep_core_q_nowcasts_alt(qcount,case_count,i1)-g_pcep_core_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_pcep_core_forecast_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_pcep_core_forecast_direction_alt(qcount,case_count,i1)=0;
	end;
	% PCE, data
	if g_pcep_core_q_metric(qcount)-g_pcep_core_q_nowcasts_alt_last(qcount,case_count,i1)>0; dfa_g_pcep_core_data_direction_alt(qcount,case_count,i1)=1;
	else; dfa_g_pcep_core_data_direction_alt(qcount,case_count,i1)=0;
	end;
	end;
end;


%---------------------------------------------------------------------
end; % case_count loop
%---------------------------------------------------------------------

%---------------------------------------------------------------------
end; % qcount loop
%---------------------------------------------------------------------

disp('-------------------------------------------------------------------------------');
disp('Case 1:');
disp('right before beginning of quarter,');
disp('no gasoline or oil price data,');
disp('missing 4 months of CPI data,');
disp('missing 4 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 2:');
disp('day 8 of the first month,');
disp('some gasoline and oil price data,');
disp('missing 4 months of CPI data,');
disp('missing 4 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 3:');
disp('day 15 of the first month,');
disp('some gasoline and oil price data');
disp('assume get CPI for last month of previous quarter on 15th (missing 3 months),');
disp('missing 4 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 4:');
disp('day 22 of the first month,');
disp('some gasoline and oil price data');
disp('missing 3 months of CPI data,');
disp('missing 4 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 5:');
disp('last day of the first month,');
disp('1 month gasoline and oil price data,');
disp('missing 3 month of CPI data,');
disp('assume get PCE for last month of previous quarter on last day (missing 3 months).');
disp('-------------------------------------------------------------------------------');
disp('Case 6:');
disp('day 8 of the second month,');
disp('1+ months gasoline and oil price data,');
disp('missing 3 months of CPI data,');
disp('missing 3 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 7:');
disp('day 15 of the second month,');
disp('1+ months gasoline and oil price data');
disp('assume get CPI for first month of current quarter on 15th (missing 2 months),');
disp('missing 3 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 8:');
disp('day 22 of the second month,');
disp('1+ months gasoline and oil price data');
disp('missing 2 months of CPI data,');
disp('missing 3 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 9:');
disp('last day of the second month,');
disp('1 month gasoline and oil price data,');
disp('missing 2 month of CPI data,');
disp('assume get PCE for first month of current quarter on last day (missing 2 months).');
disp('-------------------------------------------------------------------------------');
disp('Case 10:');
disp('day 8 of the third month,');
disp('2+ months gasoline and oil price data,');
disp('missing 2 months of CPI data,');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 11:');
disp('day 15 of the third month,');
disp('2+ months gasoline and oil price data');
disp('assume get CPI for second month of current quarter on 15th (missing 1 month),');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 12:');
disp('day 22 of the third month,');
disp('2+ months gasoline and oil price data');
disp('missing 1 month of CPI data,');
disp('missing 2 months of PCE data.');
disp('-------------------------------------------------------------------------------');
disp('Case 13:');
disp('last day of the third month,');
disp('3 months gasoline and oil price data,');
disp('missing 1 month of CPI data,');
disp('assume get PCE for second month of current quarter on last day (missing 1 month).');
disp('-------------------------------------------------------------------------------');
disp('Case 14:');
disp('middle of the first month in the next quarter,');
disp('3 months gasoline and oil price data,');
disp('assume get CPI for third month of current quarter on 15th (missing 0 months),');
disp('missing 1 month of PCE data.');
disp('-------------------------------------------------------------------------------');

%---------------------------------------------------------------------
% Directional forecast accuracy
%---------------------------------------------------------------------
pt_tests_q_cpi=NaN(3,no_cases_q);
pt_tests_q_cpi_core=NaN(3,no_cases_q);
pt_tests_q_pcep=NaN(3,no_cases_q);
pt_tests_q_pcep_core=NaN(3,no_cases_q);
for i1=1:no_cases_q;
	f1=dfa_g_cpi_forecast_direction(:,i1)~=junk_nos & dfa_g_cpi_data_direction(:,i1)~=junk_nos;
	[pt_tests_q_cpi(1,i1),pt_tests_q_cpi(2,i1),pt_tests_q_cpi(3,i1)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_forecast_direction(f1,i1),dfa_g_cpi_data_direction(f1,i1));
	f1=dfa_g_cpi_core_forecast_direction(:,i1)~=junk_nos & dfa_g_cpi_core_data_direction(:,i1)~=junk_nos;
	[pt_tests_q_cpi_core(1,i1),pt_tests_q_cpi_core(2,i1),pt_tests_q_cpi_core(3,i1)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_core_forecast_direction(f1,i1),dfa_g_cpi_core_data_direction(f1,i1));
	f1=dfa_g_pcep_forecast_direction(:,i1)~=junk_nos & dfa_g_pcep_data_direction(:,i1)~=junk_nos;
	[pt_tests_q_pcep(1,i1),pt_tests_q_pcep(2,i1),pt_tests_q_pcep(3,i1)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcep_forecast_direction(f1,i1),dfa_g_pcep_data_direction(f1,i1));
	f1=dfa_g_pcep_core_forecast_direction(:,i1)~=junk_nos & dfa_g_pcep_core_data_direction(:,i1)~=junk_nos;
	[pt_tests_q_pcep_core(1,i1),pt_tests_q_pcep_core(2,i1),pt_tests_q_pcep_core(3,i1)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcep_core_forecast_direction(f1,i1),dfa_g_pcep_core_data_direction(f1,i1));
end;
%
% for other models
pt_tests_q_cpi_alt=NaN(3,no_cases_q,no_alt_cases);
pt_tests_q_cpi_core_alt=NaN(3,no_cases_q,no_alt_cases);
pt_tests_q_pcep_alt=NaN(3,no_cases_q,no_alt_cases);
pt_tests_q_pcep_core_alt=NaN(3,no_cases_q,no_alt_cases);
for i2=1:no_alt_cases;
for i1=1:no_cases_q;
	f1=dfa_g_cpi_forecast_direction_alt(:,i1,i2)~=junk_nos & dfa_g_cpi_data_direction_alt(:,i1,i2)~=junk_nos;
	[pt_tests_q_cpi_alt(1,i1,i2),pt_tests_q_cpi_alt(2,i1,i2),pt_tests_q_cpi_alt(3,i1,i2)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_forecast_direction_alt(f1,i1,i2),dfa_g_cpi_data_direction_alt(f1,i1,i2));
	f1=dfa_g_cpi_core_forecast_direction_alt(:,i1,i2)~=junk_nos & dfa_g_cpi_core_data_direction_alt(:,i1,i2)~=junk_nos;
	[pt_tests_q_cpi_core_alt(1,i1,i2),pt_tests_q_cpi_core_alt(2,i1,i2),pt_tests_q_cpi_core_alt(3,i1,i2)]=fn_pesaran_timmermann_sc_dfa(dfa_g_cpi_core_forecast_direction_alt(f1,i1,i2),dfa_g_cpi_core_data_direction_alt(f1,i1,i2));
	f1=dfa_g_pcep_forecast_direction_alt(:,i1,i2)~=junk_nos & dfa_g_pcep_data_direction_alt(:,i1,i2)~=junk_nos;
	[pt_tests_q_pcep_alt(1,i1,i2),pt_tests_q_pcep_alt(2,i1,i2),pt_tests_q_pcep_alt(3,i1,i2)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcep_forecast_direction_alt(f1,i1,i2),dfa_g_pcep_data_direction_alt(f1,i1,i2));
	f1=dfa_g_pcep_core_forecast_direction_alt(:,i1,i2)~=junk_nos & dfa_g_pcep_core_data_direction_alt(:,i1,i2)~=junk_nos;
	[pt_tests_q_pcep_core_alt(1,i1,i2),pt_tests_q_pcep_core_alt(2,i1,i2),pt_tests_q_pcep_core_alt(3,i1,i2)]=fn_pesaran_timmermann_sc_dfa(dfa_g_pcep_core_forecast_direction_alt(f1,i1,i2),dfa_g_pcep_core_data_direction_alt(f1,i1,i2));
end;
end;

%---------------------------------------------------------------------
% Compute forecast errors
%---------------------------------------------------------------------
rmses_q_cpi=NaN*ones(1,no_cases_q);
rmses_q_cpi_core=NaN*ones(1,no_cases_q);
rmses_q_cpi_robust=NaN*ones(no_robustness_exercises,no_cases_q);
rmses_q_cpi_core_robust=NaN*ones(no_robustness_exercises,no_cases_q);
rmses_q_cpi_alt=NaN*ones(no_alt_cases,no_cases_q);
rmses_q_cpi_core_alt=NaN*ones(no_alt_cases,no_cases_q);
gwstat_sig_q_cpi_quarterly=NaN(no_alt_cases,no_cases_q);
gwstat_sig_q_cpi_core_quarterly=NaN(no_alt_cases,no_cases_q);
if pce_switch;
rmses_q_pcep=NaN*ones(1,no_cases_q);
rmses_q_pcep_core=NaN*ones(1,no_cases_q);
rmses_q_pcep_robust=NaN*ones(no_robustness_exercises,no_cases_q);
rmses_q_pcep_core_robust=NaN*ones(no_robustness_exercises,no_cases_q);
rmses_q_pcep_alt=NaN*ones(no_alt_cases,no_cases_q);
rmses_q_pcep_core_alt=NaN*ones(no_alt_cases,no_cases_q);
gwstat_sig_q_pcep_quarterly=NaN(no_alt_cases,no_cases_q);
gwstat_sig_q_pcep_core_quarterly=NaN(no_alt_cases,no_cases_q);
end;
for i1=1:no_cases_q;
	% CPI
	f1=g_cpi_q_nowcasts(:,i1);
	a1=g_cpi_q_metric;
	i2=f1~=junk_nos & a1~=junk_nos & cond1;
	r1=sqrt(mean((f1(i2)-a1(i2)).^2));
	rmses_q_cpi(i1)=r1;
	for i0=1:no_robustness_exercises;
		f2=g_cpi_q_nowcasts_robust(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_cpi_robust(i0,i1)=r1;
	end;
	for i0=1:no_alt_cases;
		f2=g_cpi_q_nowcasts_alt(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_cpi_alt(i0,i1)=r1;
		% errors
		e1=f1(i2)-a1(i2);
		e2=f2(i2)-a1(i2);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_q_cpi_quarterly(i0,i1)]=CPAtest(e1.^2-e2.^2,ones(sum(i2),1),1);
	end;
	%		
	% Core CPI
	f1=g_cpi_core_q_nowcasts(:,i1);
	a1=g_cpi_core_q_metric;
	i2=f1~=junk_nos & a1~=junk_nos & cond1;
	r1=sqrt(mean((f1(i2)-a1(i2)).^2));
	rmses_q_cpi_core(i1)=r1;
	for i0=1:no_robustness_exercises;
		f2=g_cpi_core_q_nowcasts_robust(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_cpi_core_robust(i0,i1)=r1;
	end;	
	for i0=1:no_alt_cases;	
		f2=g_cpi_core_q_nowcasts_alt(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_cpi_core_alt(i0,i1)=r1;
		% errors
		e1=f1(i2)-a1(i2);
		e2=f2(i2)-a1(i2);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_q_cpi_core_quarterly(i0,i1)]=CPAtest(e1.^2-e2.^2,ones(sum(i2),1),1);				
	end;
	%
	if pce_switch;
	% PCE
	f1=g_pcep_q_nowcasts(:,i1);
	a1=g_pcep_q_metric;
	i2=f1~=junk_nos & a1~=junk_nos & cond5 & cond1;
	r1=sqrt(mean((f1(i2)-a1(i2)).^2));
	rmses_q_pcep(i1)=r1;
	for i0=1:no_robustness_exercises;
		f2=g_pcep_q_nowcasts_robust(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond5 & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_pcep_robust(i0,i1)=r1;
	end;
	for i0=1:no_alt_cases;
		f2=g_pcep_q_nowcasts_alt(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond5 & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_pcep_alt(i0,i1)=r1;
		% errors
		e1=f1(i2)-a1(i2);
		e2=f2(i2)-a1(i2);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_q_pcep_quarterly(i0,i1)]=CPAtest(e1.^2-e2.^2,ones(sum(i2),1),1);			
	end;
	%
	% Core PCE
	f1=g_pcep_core_q_nowcasts(:,i1);
	a1=g_pcep_core_q_metric;
	i2=f1~=junk_nos & a1~=junk_nos & cond3 & cond1;
	r1=sqrt(mean((f1(i2)-a1(i2)).^2));
	rmses_q_pcep_core(i1)=r1;		
	for i0=1:no_robustness_exercises;
		f2=g_pcep_core_q_nowcasts_robust(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond5 & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_pcep_core_robust(i0,i1)=r1;
	end;
	for i0=1:no_alt_cases;
		f2=g_pcep_core_q_nowcasts_alt(:,i1,i0);
		i2=f2~=junk_nos & a1~=junk_nos & cond3 & cond1;
		r1=sqrt(mean((f2(i2)-a1(i2)).^2));
		rmses_q_pcep_core_alt(i0,i1)=r1;
		% errors
		e1=f1(i2)-a1(i2);
		e2=f2(i2)-a1(i2);
		% GW test
		[gw_teststat,gw_critval,gwstat_sig_q_pcep_core_quarterly(i0,i1)]=CPAtest(e1.^2-e2.^2,ones(sum(i2),1),1);		
	end;
	end; % pce_switch
end;
disp(['Standard deviation of quarterly CPI, metric releases, = ',num2str(std(g_cpi_q_metric(date_q>=start_quarter & date_q<=end_quarter & cond1)))]);
disp(['Standard deviation of quarterly core CPI, metric releases, = ',num2str(std(g_cpi_core_q_metric(date_q>=start_quarter & date_q<=end_quarter & cond1)))]);
disp('CPI and Core CPI nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_cpi;rmses_q_cpi_core]));
if pce_switch;
disp(['Standard deviation of quarterly PCE, metric releases, = ',num2str(std(g_pcep_q_metric(date_q>=start_quarter & date_q<=end_quarter & cond5 & cond1)))]);
disp(['Standard deviation of quarterly core PCE, metric releases, = ',num2str(std(g_pcep_core_q_metric(date_q>=start_quarter & date_q<=end_quarter & cond3 & cond1)))]);
disp('PCE and Core PCE nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_pcep;rmses_q_pcep_core]));
end; % pce_switch
disp('Alternatives, CPI nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_cpi_alt]));
disp('Alternatives, Core CPI nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_cpi_core_alt]));
if pce_switch;
disp('Alternatives, PCE nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_pcep_alt]));
disp('Alternatives, Core PCE nowcasting/forecasting');
disp(num2str([1:no_cases_q;rmses_q_pcep_core_alt]));
end; % pce_switch
%
% Robustness exercises
quarterly_robustness_out=NaN(no_robustness_exercises,6);
for i1=1:no_robustness_exercises;
	% CPI, H1 (cases 1-7)
	quarterly_robustness_out(i1,1)=mean(rmses_q_cpi_robust(i1,1:7)./rmses_q_cpi(1:7));
	% CPI, H2 (cases 8-13)
	quarterly_robustness_out(i1,2)=mean(rmses_q_cpi_robust(i1,8:13)./rmses_q_cpi(8:13));
	% Core CPI, H1 (cases 1-7)
	quarterly_robustness_out(i1,3)=mean(rmses_q_cpi_core_robust(i1,1:7)./rmses_q_cpi_core(1:7));
	% Core CPI, H2 (cases 8-13)
	quarterly_robustness_out(i1,4)=mean(rmses_q_cpi_core_robust(i1,8:13)./rmses_q_cpi_core(8:13));
	% PCE, H1 (cases 1-7)
	quarterly_robustness_out(i1,5)=mean(rmses_q_pcep_robust(i1,1:7)./rmses_q_pcep(1:7));
	% PCE, H2 (cases 8-14)
	quarterly_robustness_out(i1,6)=mean(rmses_q_pcep_robust(i1,8:14)./rmses_q_pcep(8:14));
	% Core PCE, H1 (cases 1-7)
	quarterly_robustness_out(i1,7)=mean(rmses_q_pcep_core_robust(i1,1:7)./rmses_q_pcep_core(1:7));
	% Core PCE, H2 (cases 8-14)
	quarterly_robustness_out(i1,8)=mean(rmses_q_pcep_core_robust(i1,8:14)./rmses_q_pcep_core(8:14));
end;

figure_4a=[rmses_q_cpi(1,1:13);rmses_q_cpi_alt(:,1:13)];
figure_4b=[rmses_q_cpi_core(1,1:13);rmses_q_cpi_core_alt(:,1:13)];
figure_4c=[rmses_q_pcep(1,1:14);rmses_q_pcep_alt(:,1:14)];
figure_4d=[rmses_q_pcep_core(1,1:14);rmses_q_pcep_core_alt(:,1:14)];

table_9=quarterly_robustness_out(1:18,:);