% nowcasting_inflation_setup.m
% Script to set up the data.

close all;
clear all;

global junk_nos tau tau_L J_ sa_years extra_months;

%---------------------------------------------------------------------
% Final complete month and quarter with data
%---------------------------------------------------------------------
last_quarter=2015.2; % final quarter for which have data
last_month=2015.06; % final month for which have data

%---------------------------------------------------------------------
% Data subject to revisions, both CPI and PCE
%---------------------------------------------------------------------
last_quarter_with_revisions=2014.4; % final quarter that has been subject to revisions--note, this will likely be determined by CPI,
																		% which is revised at the beginning of the year--choose the earlier of the series
last_month_with_revisions=2014.12; % final month that has been subject to revisions, again based on CPI
first_quarter_for_revisions=2000.2; % first quarter in which have both CPI and PCE real-time data
first_month_for_revisions=2000.06; % first month in which have both CPI and PCE real-time data

%---------------------------------------------------------------------
% Toggle to exclude 2008Q4 from the quarterly exercise only
%---------------------------------------------------------------------
% In the fourth quarter of 2008, inflation plunged: CPI inflation fell from
% +7% in Q3 to -9% in Q4.  In the quarterly exercise with competing forecasting models,
% the competing models utterly fail to predict the deflation in Q4, with absolute
% forecast errors in the range of 15 percentage points.  With a short sample,
% this extreme outlier can bias the results.  This toggle allows you to drop
% this quarter from this exercise ONLY!  Note that when doing other comparisons,
% we will keep 2008Q4 in, as professional forecasters should have had an advantage
% in handling the situation versus pure statistical forecasts. 
exclude_2008q4_switch=1; % toggle is on, exclude 2008Q4 from quarterly exercise; see cond1 below
%exclude_2008q4_switch=0; % toggle is off, include 2008Q4 in quarterly exercise

%---------------------------------------------------------------------
% Toggle to exclude September 2001 effects from core PCE RMSEs
%---------------------------------------------------------------------
% Core PCE inflation dropped sharply in September 2001 and rebounded 
% sharply in October 2001.  These two months alone skew the entire
% time series.  Movements in headline PCE were not incredibly unusual,
% but clearly by construction they will be affected as well.  By 
% comparison, the movements in CPI and core CPI were typical (core CPI did not
% fall in September 2001, and thus it did not have the same rebound,
% either).  This switch allows you to exclude September 2001 and 
% October 2001 from the monthly computation of RMSEs for core PCE
% and/or headline PCE.  You can also exclude the quarterly computation of RMSEs for
% core PCE and headline PCE for 2001Q3 and 2001Q4.
pce_core_2001_switch_m=1; % 9/11 toggle is on for monthly data, exclude 9/2001 and 10/2001; see cond2 below
%pce_core_2001_switch_m=0; % 9/11 toggle is off for monthly data, include 9/2001 and 10/2001
pce_core_2001_switch_q=1; % 9/11 toggle is on for quarterly data, exclude 2001.3 and 2001.4; see cond3 below
%pce_core_2001_switch_q=0; % 9/11 toggle is off for quarterly data, include 2001.3 and 2001.4
pce_2001_switch_m=1; % 9/11 toggle is on for monthly data, exclude 9/2001 and 10/2001; see cond4 below
%pce_2001_switch_m=0;% 9/11 toggle is off for monthly data, include 9/2001 and 10/2001
pce_2001_switch_q=1; % 9/11 toggle is on for quarterly data, exclude 2001.3 and 2001.4; see cond5 below
%pce_2001_switch_q=0; % 9/11 toggle is off for quarterly data, include 2001.3 and 2001.4

%---------------------------------------------------------------------
% How far into the future to go when computing previous inflation
%---------------------------------------------------------------------
h=2; % number of months into the future to go to look back and compute inflation
		 % h=0 means use first release

%---------------------------------------------------------------------
% Toggle for PCE inflation
%---------------------------------------------------------------------
% PCE has a slightly shorter real-time sample than CPI, this allows to
% just conduct CPI analysis if desired
pce_switch=1; % PCE toggle is on, compute PCE nowcasts
% pce_switch=0; % PCE toggle is off, do not compute PCE nowcasts

%---------------------------------------------------------------------
% Months to use for monthly nowcasting analysis
%---------------------------------------------------------------------
% month to start the analysis
start_month=2000.09; % real-time PCE starts 8/2000
% start_month=1999.03; % real-time CPI starts 2/1999
% month to end the analysis
end_month=2015.06;

%---------------------------------------------------------------------
% Months to use for monthly nowcasting analysis of year-over-year CPI, Modugno comparison
%---------------------------------------------------------------------
% month to start the analysis
start_month_yoy_modugno=2001.01; % per Modugno
% month to end the analysis
end_month_yoy_modugno=2011.12; % per Modugno 

%---------------------------------------------------------------------
% Quarters to use for quarterly nowcasting analysis
%---------------------------------------------------------------------
% quarter to start the analysis
start_quarter=2000.4; % real-time PCE starts 8/2000
% start_quarter=1999.2; % real-time CPI starts 2/1999
% quarter to end the analysis
end_quarter=2015.2;

%---------------------------------------------------------------------
% Toggle for conducting pseudo real-time analysis as of a given date
%---------------------------------------------------------------------
%final_data_switch=1; % Toggle for pseudo real-time analysis with final data
final_data_switch=0; % Toggle for pseudo real-time analysis is off, use actual real-time data

%---------------------------------------------------------------------
% Quarters to use for Blue Chip comparison (CPI only)
%---------------------------------------------------------------------
% quarter to start the analysis
start_quarter_bcei=1999.2; % real-time CPI starts 2/1999
% quarter to end the analysis
end_quarter_bcei=2015.2;

%---------------------------------------------------------------------
% Quarters to use for Survey of Professional Forecasters
%---------------------------------------------------------------------
% quarter to start the analysis
start_quarter_spf=1999.2; % SPF core CPI, PCE, and core PCE start 2007.1
% quarter to end the analysis
end_quarter_spf=2015.2;

%---------------------------------------------------------------------
% Quarters to use for Greenbook
%---------------------------------------------------------------------
% quarter to start the analysis
start_quarter_gb=1999.2; % PCE and core PCE start 2000.1
% quarter to end the analysis
end_quarter_gb=2009.4;






%---------------------------------------------------------------------
% Definitions
%---------------------------------------------------------------------
% number instead of NaNs
junk_nos=1e10; % compatible with other files
%
% number of monthly observations for moving averages (=12 in spirit of Atkeson/Ohanian)
J_=12;
%
% length of window to estimate model, months
tau=24; 
%
% length of window to estimate the oil-gasoline error correction model, months
tau_L=60;
%
% number of years to use for computing seasonal factors
sa_years=3;
%
% number of months over which gasoline prices return to the last observed oil price
extra_months=1;

%---------------------------------------------------------------------
% Load data
%---------------------------------------------------------------------
% Load commodity data.
[p_commodities,p_commodities_txt]=xlsread('nowcast_inflation_data','commodity_prices');
p_commodities_dates=p_commodities_txt(2:end,1);
p_commodities_dates=p_commodities_dates(1:size(p_commodities,1));
p_commodities_dates=str2num(char(p_commodities_dates));
i1=p_commodities_dates<(last_month+.01)*10000; % remove entries beyond the end of the sample
p_commodities_dates=p_commodities_dates(i1);
p_commodities=p_commodities(i1,:);
%
% Oil prices.
% Column 1 is Brent crude oil prices from Financial Times:
p_oil_brent_ft_d_rt=[p_commodities_dates,p_commodities(:,1)];
% Column 3 is WTI crude oil prices from EIA:
p_oil_wti_d_rt=[p_commodities_dates,p_commodities(:,3)];
% 
% For our model, pick "the" oil price
p_oil_d_rt=p_oil_brent_ft_d_rt;
%
% Foodstuffs prices. Column 4 is CRB spot commodity price index: foodstuffs.
p_foodstuffs_d_rt=[p_commodities_dates,p_commodities(:,4)];
% For a single matrix with all commodity prices, with Brent crude oil first and 
% all other commodities, Treasury rates, the S&P500, and the dollar:
p_all_commodities_d_rt=[p_commodities_dates,p_commodities(:,[1,4:16])];
%
% Load retail gasoline prices.  First column will be dates, second column
% will be prices (data are real-time by construction).
[p_gasoline,p_gasoline_txt]=xlsread('nowcast_inflation_data','gasoline_prices');
p_gasoline_dates=p_gasoline(:,1);
i1=p_gasoline_dates<(last_month+.01)*10000; % remove entries beyond the end of the sample
p_gasoline_dates=p_gasoline_dates(i1);
p_gasoline_w_rt=[p_gasoline_dates,p_gasoline(i1,2)];
% For a detailed breakdown of gasoline prices, use columns 2 through 5; in order, these are:
% regular, midgrade, premium, and diesel
p_gasoline_by_grade_w_rt=[p_gasoline_dates,p_gasoline(i1,3:6)];
%
% Other real-time data.
% Convention: a matrix.  
% First column, rows 2:R provides the dates in time-series format; e.g., 2012.01 for January 2012.
% First row, columns 2:C provides the date of the release; e.g., 20120103 for data released on January 3, 2012.
% Time series are oriented vertically.
% Load real-time PCE data.
[pcepm_rt]=xlsread('nowcast_inflation_data','pcepm_raw');
% Load real-time core PCE data.
[pcepm_core_rt]=xlsread('nowcast_inflation_data','pcepm_core_raw');
% Load real-time CPI data.
[cpi_rt]=xlsread('nowcast_inflation_data','cpi_raw');
% Load real-time core CPI data.
[cpi_core_rt]=xlsread('nowcast_inflation_data','cpi_core_raw');
% Load real-time CPI food data.
[cpi_food_rt]=xlsread('nowcast_inflation_data','cpi_food_raw');
% Load real-time CPI gasoline data.
[cpi_gasoline_rt]=xlsread('nowcast_inflation_data','cpi_gasoline_raw');
% Load NSA CPI data.
[nsa_data]=xlsread('nowcast_inflation_data','cpi_nsa_raw');
cpi_nsa_rt=nsa_data(:,2);
cpi_core_nsa_rt=nsa_data(:,3);
%%%%% cpi_food_nsa_rt=nsa_data(:,4);
%%%%% cpi_gasoline_nsa_rt=nsa_data(:,5);
%%%%% cpi_energy_nsa_rt=nsa_data(:,6); % from Modugno (2013)
%%%%% cpi_food_and_beverage_nsa_rt=nsa_data(:,7); % from Modugno (2013)
%%%%% cpi_housing_nsa_rt=nsa_data(:,8); % from Modugno (2013)
%%%%% cpi_other_goods_and_services_nsa_rt=nsa_data(:,9); % from Modugno (2013)
%%%%% cpi_transportation_nsa_rt=nsa_data(:,10); % from Modugno (2013)
%%%%% ism_mfg_prices_nsa_rt=nsa_data(:,11); % from Modugno (2013)
%%%%% ism_nonmfg_prices_nsa_rt=nsa_data(:,12); % from Modugno (2013)

%---------------------------------------------------------------------
% There are some anamolies in the data.
%---------------------------------------------------------------------
% In some cases, the real-time CPI datasets contain an extra reading when seasonal factors are 
% revised;  these new seasonals have not always coincided with the release of new data.  For 
% example, on January 16, 2013, the BLS released the December 2012 CPI.  On February 19, 2013, the 
% BLS released new seasonally adjusted data for the CPI through December 2012.  On February 21, 2013, 
% the BLS released the January 2013 CPI.  A true real-time horserace may require keeping both 
% vintages, if the date is the key way that time is being measured.  But if the arrival of a new 
% release is how the passing of time is being measured, then the new seasonal factors release can be
% deleted.  For the CPI, the time between the new seasonally adjusted data being released and the new
% release is a couple of days.  Additionally, this pattern is not replicated throughout the dataset,
% which makes it unclear if this pattern has always occurred (and the real-time dataset is missing 
% it) or it is a new phenomenon.  
% To clean the data for the monthly and quarterly baseline exercises, we remove the vintages that only 
% contain new seasonal factors and are not part of a new release.  This appears to affect the CPI 
% sources from 2005 onward.
% The real-time CPI series also have different lengths--some have longer coverage than others (i.e.,
% there are more vintages going farther back in time).  We shorten the longer ones to match the 
% shortest series.
[cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt]=fn_clean_cpi(cpi_rt,cpi_core_rt,cpi_food_rt,cpi_gasoline_rt);
% The real-time PCE dataset for headline and core PCE price indexes contains very few entries for
% the release that occurred on December 23, 2003; a more complete history is available as of 
% January 5, 2004.  We delete the December 23, 2003, PCE and core PCE vintages; they are too short
% to be useful.  Additionally, the August 4, 2009 PCE history is limited, while the full history was
% released August 5, 2009.  We delete the August 4, 2009, PCE and core PCE vintages.
[pcepm_rt,pcepm_core_rt]=fn_clean_pce(pcepm_rt,pcepm_core_rt);

%---------------------------------------------------------------------
% Set up date_m and date_q for general use
%---------------------------------------------------------------------
date_m=cpi_rt(2:end,1);
date_q=repmat([1947:round(last_quarter)],4,1)+repmat([0.1;0.2;0.3;0.4],1,round(last_quarter)-1947+1);
date_q=reshape(date_q,prod(size(date_q)),1);
date_q=date_q(date_q<=last_quarter);
date_w=p_gasoline_dates;
date_d=p_commodities_dates;

%---------------------------------------------------------------------
% Set up junk_nos_m
%---------------------------------------------------------------------
junk_nos_m=junk_nos*ones(size(date_m));

%---------------------------------------------------------------------
% Set up junk_nos_q
%---------------------------------------------------------------------
junk_nos_q=junk_nos*ones(size(date_q));

%---------------------------------------------------------------------
% Replace NaNs with junk_nos
%---------------------------------------------------------------------
p_oil_d_rt(isnan(p_oil_d_rt))=junk_nos;
p_oil_wti_d_rt(isnan(p_oil_wti_d_rt))=junk_nos;
p_foodstuffs_d_rt(isnan(p_foodstuffs_d_rt))=junk_nos;
p_gasoline_w_rt(isnan(p_gasoline_w_rt))=junk_nos;
cpi_rt(isnan(cpi_rt))=junk_nos;
cpi_core_rt(isnan(cpi_core_rt))=junk_nos;
cpi_food_rt(isnan(cpi_food_rt))=junk_nos;
cpi_gasoline_rt(isnan(cpi_gasoline_rt))=junk_nos;
pcepm_rt(isnan(pcepm_rt))=junk_nos;
pcepm_core_rt(isnan(pcepm_core_rt))=junk_nos;
cpi_nsa_rt(isnan(cpi_nsa_rt))=junk_nos;
cpi_core_nsa_rt(isnan(cpi_core_nsa_rt))=junk_nos;
%%%%%
%%%%% cpi_food_nsa_rt(isnan(cpi_food_nsa_rt))=junk_nos;
%%%%% cpi_gasoline_nsa_rt(isnan(cpi_gasoline_nsa_rt))=junk_nos;
%%%%% p_all_commodities_d_rt(isnan(p_all_commodities_d_rt))=junk_nos;
%%%%% p_gasoline_by_grade_w_rt(isnan(p_gasoline_by_grade_w_rt))=junk_nos;
%%%%% cpi_energy_nsa_rt(isnan(cpi_energy_nsa_rt))=junk_nos;
%%%%% cpi_food_and_beverage_nsa_rt(isnan(cpi_food_and_beverage_nsa_rt))=junk_nos;
%%%%% cpi_housing_nsa_rt(isnan(cpi_housing_nsa_rt))=junk_nos;
%%%%% cpi_other_goods_and_services_nsa_rt(isnan(cpi_other_goods_and_services_nsa_rt))=junk_nos;
%%%%% cpi_transportation_nsa_rt(isnan(cpi_transportation_nsa_rt))=junk_nos;
%%%%% ism_mfg_prices_nsa_rt(isnan(ism_mfg_prices_nsa_rt))=junk_nos;
%%%%% ism_nonmfg_prices_nsa_rt(isnan(ism_nonmfg_prices_nsa_rt))=junk_nos;

%---------------------------------------------------------------------
% Allow for doing the exercise with revised/final data, as of a given
% date; note: final_date should be in yyyymmdd format
%---------------------------------------------------------------------
if final_data_switch;
cpi_rt=fn_use_final_data(cpi_rt,final_date);
cpi_core_rt=fn_use_final_data(cpi_core_rt,final_date);
cpi_food_rt=fn_use_final_data(cpi_food_rt,final_date);
cpi_gasoline_rt=fn_use_final_data(cpi_gasoline_rt,final_date);
pcepm_rt=fn_use_final_data(pcepm_rt,final_date);
pcepm_core_rt=fn_use_final_data(pcepm_core_rt,final_date);
% Check to ensure exercises do not require data available beyond the final_date
fd=(round(final_date/100))/100;
i1=find(date_m==fd);
if end_month>=date_m(i1-2); % need to ensure you have final CPI and PCE readings
	end_month=date_m(i1-2);
	disp('Variable end_month changed during course of creating pseudo real-time data');
	disp(['New end_month=',num2str(end_month)]);
end;
if round(end_quarter)+3*10*(end_quarter-round(end_quarter))/100>date_m(i1-2);
	i2=(i1-2)-rem(round(100*rem(date_m(i1-2),1)),3);
	fdyyyy=round(date_m(i2));
	fdq=round(100*(date_m(i2)-round(date_m(i2))))/30;
	end_quarter=fdyyyy+fdq;
	disp('Variable end_quarter changed during course of creating pseudo real-time data');
	disp(['New end_quarter=',num2str(end_quarter)]);
end;
end;

%---------------------------------------------------------------------
% Compute the first reported (initial) inflation reading
%---------------------------------------------------------------------
% Monthly--use simple (non-annualized) percent change
g_cpi_initial=fn_compute_initial_m(date_m,cpi_rt);
g_cpi_core_initial=fn_compute_initial_m(date_m,cpi_core_rt);
g_pcepm_initial=fn_compute_initial_m(date_m,pcepm_rt);
g_pcepm_core_initial=fn_compute_initial_m(date_m,pcepm_core_rt);
% Quarterly--use annualized rate of change
g_pcep_q_initial=fn_compute_initial_q(date_q,date_m,pcepm_rt);
g_pcep_core_q_initial=fn_compute_initial_q(date_q,date_m,pcepm_core_rt);
g_cpi_q_initial=fn_compute_initial_q(date_q,date_m,cpi_rt);
g_cpi_core_q_initial=fn_compute_initial_q(date_q,date_m,cpi_core_rt);

%---------------------------------------------------------------------
% For the sake of comparison, compute the final (last available)
% inflation readings
%---------------------------------------------------------------------
% Monthly--use simple (non-annualized) percent change
g_cpi_final=fn_compute_g_monthly(cpi_rt(2:end,end));
g_cpi_core_final=fn_compute_g_monthly(cpi_core_rt(2:end,end));
g_pcepm_final=fn_compute_g_monthly(pcepm_rt(2:end,end));
g_pcepm_core_final=fn_compute_g_monthly(pcepm_core_rt(2:end,end));
% Quarterly--use annualized rate of change
[tmp,g_pcep_q_final]=fn_mtoqavg(pcepm_rt(2:end,end),date_m,date_q);
[tmp,g_pcep_core_q_final]=fn_mtoqavg(pcepm_core_rt(2:end,end),date_m,date_q);
[tmp,g_cpi_q_final]=fn_mtoqavg(cpi_rt(2:end,end),date_m,date_q);
[tmp,g_cpi_core_q_final]=fn_mtoqavg(cpi_core_rt(2:end,end),date_m,date_q);

%---------------------------------------------------------------------
% Compute the metric to test the model
%---------------------------------------------------------------------
% Monthly--use simple (non-annualized) percent change
[g_cpi_metric,g_cpi_metric_yoy,g_cpi_metric_dln,g_cpi_metric_yoy_dln]=fn_compute_metric_m(date_m,cpi_rt,h);
[g_cpi_core_metric,g_cpi_core_metric_yoy,g_cpi_core_metric_dln,g_cpi_core_metric_yoy_dln]=fn_compute_metric_m(date_m,cpi_core_rt,h);
[g_pcepm_metric,g_pcepm_metric_yoy,g_pcepm_metric_dln,g_pcepm_metric_yoy_dln]=fn_compute_metric_m(date_m,pcepm_rt,h);
[g_pcepm_core_metric,g_pcepm_core_metric_yoy,g_pcepm_core_metric_dln,g_pcepm_core_metric_yoy_dln]=fn_compute_metric_m(date_m,pcepm_core_rt,h);
% Quarterly--use annualized rate of change
g_pcep_q_metric=fn_compute_metric_q(date_q,date_m,pcepm_rt,h);
g_pcep_core_q_metric=fn_compute_metric_q(date_q,date_m,pcepm_core_rt,h);
g_cpi_q_metric=fn_compute_metric_q(date_q,date_m,cpi_rt,h);
g_cpi_core_q_metric=fn_compute_metric_q(date_q,date_m,cpi_core_rt,h);
% Year-over-year metric for NSA CPI is special
g_cpi_nsa_metric_yoy=junk_nos*ones(size(date_m));
g_cpi_nsa_metric_yoy_dln=junk_nos*ones(size(date_m));
g_cpi_core_nsa_metric_yoy=junk_nos*ones(size(date_m));
i1=min(find(cpi_nsa_rt~=junk_nos));
i2=max(find(cpi_nsa_rt~=junk_nos));
for i3=i1+12:i2;
	g_cpi_nsa_metric_yoy(i3)=100*(cpi_nsa_rt(i3)/cpi_nsa_rt(i3-12)-1);
	g_cpi_nsa_metric_yoy_dln(i3)=100*log(cpi_nsa_rt(i3)/cpi_nsa_rt(i3-12));
	g_cpi_core_nsa_metric_yoy(i3)=100*(cpi_core_nsa_rt(i3)/cpi_core_nsa_rt(i3-12)-1);
end;

%---------------------------------------------------------------------
% Exclusions
%---------------------------------------------------------------------
% Determine monthly exclusions
if pce_core_2001_switch_m; % excluding them
	cond2=date_m~=2001.09 & date_m~=2001.10;
else;
	cond2=ones(size(date_m));
end;
if pce_2001_switch_m; 
	cond4=date_m~=2001.09 & date_m~=2001.10;
else;
	cond4=ones(size(date_m));
end;
% Exclude November 2001 as well?
%if pce_core_2001_switch_m & pce_2001_switch_m;
%	cond6=date_m~=2001.11;
%else;
	cond6=ones(size(date_m));
%end;
% Determine quarterly exclusions
if exclude_2008q4_switch; % excluding it
	cond1=date_q~=2008.4;
else; % including it
	cond1=ones(size(date_q));
end;
if pce_core_2001_switch_q; 
	cond3=date_q~=2001.3 & date_q~=2001.4;
else;
	cond3=ones(size(date_q));
end;
if pce_2001_switch_q;
	cond5=date_q~=2001.3 & date_q~=2001.4;
else;
	cond5=ones(size(date_q));
end;